import os
import zipfile
import logging
from datetime import datetime
from typing import List, Dict, Tuple, Literal

import gradio as gr

from kotonebot.backend.context import task_registry
from kotonebot.config.manager import load_config, save_config
from kotonebot.tasks.common import (
    BaseConfig, APShopItems, PurchaseConfig, ActivityFundsConfig,
    PresentsConfig, AssignmentConfig, ContestConfig, ProduceConfig,
    MissionRewardConfig, PIdol, DailyMoneyShopItems
)
from kotonebot.config.base_config import UserConfig, BackendConfig
from kotonebot.run.run import initialize, start, execute

# 初始化日志
os.makedirs('logs', exist_ok=True)
log_formatter = logging.Formatter('[%(asctime)s][%(levelname)s][%(name)s] %(message)s')
log_filename = datetime.now().strftime('logs/%y-%m-%d-%H-%M-%S.log')

console_handler = logging.StreamHandler()
console_handler.setFormatter(log_formatter)
file_handler = logging.FileHandler(log_filename, encoding='utf-8')
file_handler.setFormatter(log_formatter)

root_logger = logging.getLogger()
root_logger.setLevel(logging.INFO)
root_logger.addHandler(console_handler)
root_logger.addHandler(file_handler)

logging.getLogger("kotonebot").setLevel(logging.DEBUG)

class KotoneBotUI:
    def __init__(self) -> None:
        self.is_running: bool = False
        self._load_config()
        self._setup_kaa()
        
    def _setup_kaa(self) -> None:
        initialize('kotonebot.tasks')
        from kotonebot.backend.debug.vars import debug
        if self.current_config.keep_screenshots:
            debug.auto_save_to_folder = 'dumps'
            debug.enabled = True
        else:
            debug.auto_save_to_folder = None
            debug.enabled = False

    def export_dumps(self) -> str:
        """导出 dumps 文件夹为 zip 文件"""
        if not os.path.exists('dumps'):
            return "dumps 文件夹不存在"
        
        timestamp = datetime.now().strftime('%y-%m-%d-%H-%M-%S')
        zip_filename = f'dumps-{timestamp}.zip'
        
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED, compresslevel=9) as zipf:
            for root, dirs, files in os.walk('dumps'):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, 'dumps')
                    zipf.write(file_path, arcname)
        
        return f"已导出到 {zip_filename}"

    def export_logs(self) -> str:
        """导出 logs 文件夹为 zip 文件"""
        if not os.path.exists('logs'):
            return "logs 文件夹不存在"
        
        timestamp = datetime.now().strftime('%y-%m-%d-%H-%M-%S')
        zip_filename = f'logs-{timestamp}.zip'
        
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED, compresslevel=9) as zipf:
            for root, dirs, files in os.walk('logs'):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, 'logs')
                    zipf.write(file_path, arcname)
        
        return f"已导出到 {zip_filename}"

    def get_button_status(self) -> str:
        if not hasattr(self, 'run_status'):
            return "启动"
        
        if not self.run_status.running:
            self.is_running = False
            return "启动"
        return "停止"

    def update_task_status(self) -> List[List[str]]:
        status_list: List[List[str]] = []
        if not hasattr(self, 'run_status'):
            for task_name, task in task_registry.items():
                status_list.append([task.name, "等待中"])
            return status_list
        
        for task_status in self.run_status.tasks:
            status_text = {
                'pending': '等待中',
                'running': '运行中',
                'finished': '已完成',
                'error': '出错',
                'cancelled': '已取消'
            }.get(task_status.status, '未知')
            status_list.append([task_status.task.name, status_text])
        return status_list

    def toggle_run(self) -> Tuple[str, List[List[str]]]:
        if not self.is_running:
            return self.start_run()
        return self.stop_run()
    
    def start_run(self) -> Tuple[str, List[List[str]]]:
        self.is_running = True
        initialize('kotonebot.tasks')
        self.run_status = start(config_type=BaseConfig)
        return "停止", self.update_task_status()

    def stop_run(self) -> Tuple[str, List[List[str]]]:
        self.is_running = False
        self.run_status.interrupt()
        return "启动", self.update_task_status()

    def save_settings(
        self,
        adb_ip: str,
        adb_port: int,
        screenshot_method: Literal['adb', 'adb_raw', 'uiautomator2'],
        keep_screenshots: bool,
        purchase_enabled: bool,
        money_enabled: bool,
        ap_enabled: bool,
        ap_items: List[str],
        money_items: List[DailyMoneyShopItems],
        activity_funds_enabled: bool,
        presents_enabled: bool,
        assignment_enabled: bool,
        mini_live_reassign: bool,
        mini_live_duration: Literal[4, 6, 12],
        online_live_reassign: bool,
        online_live_duration: Literal[4, 6, 12],
        contest_enabled: bool,
        produce_enabled: bool,
        produce_mode: Literal["regular"],
        produce_count: int,
        produce_idols: List[str],
        auto_set_memory: bool,
        auto_set_support: bool,
        use_pt_boost: bool,
        use_note_boost: bool,
        follow_producer: bool,
        mission_reward_enabled: bool
    ) -> str:
        ap_items_enum: List[Literal[0, 1, 2, 3]] = []
        ap_items_map: Dict[str, APShopItems] = {
            "支援强化点数提升": APShopItems.PRODUCE_PT_UP,
            "笔记数提升": APShopItems.PRODUCE_NOTE_UP,
            "重新挑战券": APShopItems.RECHALLENGE,
            "回忆再生成券": APShopItems.REGENERATE_MEMORY
        }
        for item in ap_items:
            if item in ap_items_map:
                ap_items_enum.append(ap_items_map[item].value)  # type: ignore
        
        self.current_config.backend.adb_ip = adb_ip
        self.current_config.backend.adb_port = adb_port
        self.current_config.backend.screenshot_impl = screenshot_method
        self.current_config.keep_screenshots = keep_screenshots
        
        options = BaseConfig(
            purchase=PurchaseConfig(
                enabled=purchase_enabled,
                money_enabled=money_enabled,
                money_items=money_items,
                ap_enabled=ap_enabled,
                ap_items=ap_items_enum
            ),
            activity_funds=ActivityFundsConfig(
                enabled=activity_funds_enabled
            ),
            presents=PresentsConfig(
                enabled=presents_enabled
            ),
            assignment=AssignmentConfig(
                enabled=assignment_enabled,
                mini_live_reassign_enabled=mini_live_reassign,
                mini_live_duration=mini_live_duration,
                online_live_reassign_enabled=online_live_reassign,
                online_live_duration=online_live_duration
            ),
            contest=ContestConfig(
                enabled=contest_enabled
            ),
            produce=ProduceConfig(
                enabled=produce_enabled,
                mode=produce_mode,
                produce_count=produce_count,
                idols=[PIdol[idol] for idol in produce_idols],
                auto_set_memory=auto_set_memory,
                auto_set_support_card=auto_set_support,
                use_pt_boost=use_pt_boost,
                use_note_boost=use_note_boost,
                follow_producer=follow_producer
            ),
            mission_reward=MissionRewardConfig(
                enabled=mission_reward_enabled
            )
        )
        
        self.current_config.options = options
        
        try:
            save_config(self.config, "config.json")
            gr.Success("设置已保存，请重启程序！")
            return ""
        except Exception as e:
            gr.Warning(f"保存设置失败：{str(e)}")
            return ""

    def _create_status_tab(self) -> None:
        with gr.Tab("状态"):
            gr.Markdown("## 状态")
            progress_bar = gr.Progress()
            
            with gr.Row():
                run_btn = gr.Button("启动", scale=1)
                debug_btn = gr.Button("调试", scale=1)
            
            task_status = gr.Dataframe(
                headers=["任务", "状态"],
                value=self.update_task_status(),
                label="任务状态"
            )
            
            def on_run_click(evt: gr.EventData) -> Tuple[str, List[List[str]]]:
                return self.toggle_run()
            
            run_btn.click(
                fn=on_run_click,
                outputs=[run_btn, task_status]
            )

            # 添加定时器，分别更新按钮状态和任务状态
            gr.Timer(1.0).tick(
                fn=self.get_button_status,
                outputs=[run_btn]
            )
            gr.Timer(1.0).tick(
                fn=self.update_task_status,
                outputs=[task_status]
            )

    def _create_task_tab(self) -> None:
        with gr.Tab("任务"):
            gr.Markdown("## 执行任务")
            
            # 创建任务选择下拉框
            task_choices = [task.name for task in task_registry.values()]
            task_dropdown = gr.Dropdown(
                choices=task_choices,
                label="选择要执行的任务",
                info="选择一个要单独执行的任务",
                type="value",
                value=None
            )
            
            # 创建执行按钮
            execute_btn = gr.Button("执行任务")
            task_result = gr.Markdown("")
            
            # TODO: 实现任务执行逻辑
            def execute_single_task(task_name: str) -> str:
                if not task_name:
                    gr.Warning("请先选择一个任务")
                    return ""
                task = None
                for name, task in task_registry.items():
                    if name == task_name:
                        task = task
                        break
                if task is None:
                    gr.Warning(f"任务 {task_name} 未找到")
                    return ""
                gr.Info(f"任务 {task_name} 开始执行。执行结束前，请勿重复点击执行。")
                execute(task, config_type=BaseConfig)
                gr.Success(f"任务 {task_name} 执行完毕")
                return ""
            
            execute_btn.click(
                fn=execute_single_task,
                inputs=[task_dropdown],
                outputs=[task_result]
            )

    def _create_purchase_settings(self) -> Tuple[gr.Checkbox, gr.Checkbox, gr.Checkbox, gr.Dropdown, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 商店购买设置")
            purchase_enabled = gr.Checkbox(
                label="启用商店购买",
                value=self.current_config.options.purchase.enabled,
                info=PurchaseConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.purchase.enabled) as purchase_group:
                money_enabled = gr.Checkbox(
                    label="启用金币购买",
                    value=self.current_config.options.purchase.money_enabled,
                    info=PurchaseConfig.model_fields['money_enabled'].description
                )
                
                # 添加金币商店商品选择
                money_items = gr.Dropdown(
                    multiselect=True,
                    choices=list(DailyMoneyShopItems.all()),
                    value=self.current_config.options.purchase.money_items,
                    label="金币商店购买物品",
                    info=PurchaseConfig.model_fields['money_items'].description
                )
                
                ap_enabled = gr.Checkbox(
                    label="启用AP购买",
                    value=self.current_config.options.purchase.ap_enabled,
                    info=PurchaseConfig.model_fields['ap_enabled'].description
                )
                
                # 转换枚举值为显示文本
                selected_items: List[str] = []
                ap_items_map = {
                    APShopItems.PRODUCE_PT_UP: "支援强化点数提升",
                    APShopItems.PRODUCE_NOTE_UP: "笔记数提升",
                    APShopItems.RECHALLENGE: "重新挑战券",
                    APShopItems.REGENERATE_MEMORY: "回忆再生成券"
                }
                for item_value in self.current_config.options.purchase.ap_items:
                    item_enum = APShopItems(item_value)
                    if item_enum in ap_items_map:
                        selected_items.append(ap_items_map[item_enum])
                
                ap_items = gr.Dropdown(
                    multiselect=True,
                    choices=list(ap_items_map.values()),
                    value=selected_items,
                    label="AP商店购买物品",
                    info=PurchaseConfig.model_fields['ap_items'].description
                )
            
            purchase_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[purchase_enabled],
                outputs=[purchase_group]
            )
        return purchase_enabled, money_enabled, ap_enabled, ap_items, money_items

    def _create_work_settings(self) -> Tuple[gr.Checkbox, gr.Checkbox, gr.Dropdown, gr.Checkbox, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 工作设置")
            assignment_enabled = gr.Checkbox(
                label="启用工作",
                value=self.current_config.options.assignment.enabled,
                info=AssignmentConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.assignment.enabled) as work_group:
                with gr.Row():
                    with gr.Column():
                        mini_live_reassign = gr.Checkbox(
                            label="启用重新分配 MiniLive",
                            value=self.current_config.options.assignment.mini_live_reassign_enabled,
                            info=AssignmentConfig.model_fields['mini_live_reassign_enabled'].description
                        )
                        mini_live_duration = gr.Dropdown(
                            choices=[4, 6, 12],
                            value=self.current_config.options.assignment.mini_live_duration,
                            label="MiniLive 工作时长",
                            interactive=True,
                            info=AssignmentConfig.model_fields['mini_live_duration'].description
                        )
                    with gr.Column():
                        online_live_reassign = gr.Checkbox(
                            label="启用重新分配 OnlineLive",
                            value=self.current_config.options.assignment.online_live_reassign_enabled,
                            info=AssignmentConfig.model_fields['online_live_reassign_enabled'].description
                        )
                        online_live_duration = gr.Dropdown(
                            choices=[4, 6, 12],
                            value=self.current_config.options.assignment.online_live_duration,
                            label="OnlineLive 工作时长",
                            interactive=True,
                            info=AssignmentConfig.model_fields['online_live_duration'].description
                        )
            
            assignment_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[assignment_enabled],
                outputs=[work_group]
            )
        return assignment_enabled, mini_live_reassign, mini_live_duration, online_live_reassign, online_live_duration

    def _create_produce_settings(self) -> Tuple[gr.Checkbox, gr.Dropdown, gr.Number, gr.Dropdown, gr.Checkbox, gr.Checkbox, gr.Checkbox, gr.Checkbox, gr.Checkbox]:
        with gr.Column():
            gr.Markdown("### 培育设置")
            produce_enabled = gr.Checkbox(
                label="启用培育",
                value=self.current_config.options.produce.enabled,
                info=ProduceConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.produce.enabled) as produce_group:
                produce_mode = gr.Dropdown(
                    choices=["regular", "pro"],
                    value=self.current_config.options.produce.mode,
                    label="培育模式",
                    info=ProduceConfig.model_fields['mode'].description
                )
                produce_count = gr.Number(
                    minimum=1,
                    value=self.current_config.options.produce.produce_count,
                    label="培育次数",
                    interactive=True,
                    info=ProduceConfig.model_fields['produce_count'].description
                )
                # 添加偶像选择
                idol_choices = [idol.name for idol in PIdol]
                selected_idols = [idol.name for idol in self.current_config.options.produce.idols]
                produce_idols = gr.Dropdown(
                    choices=idol_choices,
                    value=selected_idols,
                    label="选择要培育的偶像",
                    multiselect=True,
                    interactive=True,
                    info=ProduceConfig.model_fields['idols'].description
                )
                auto_set_memory = gr.Checkbox(
                    label="自动编成回忆",
                    value=self.current_config.options.produce.auto_set_memory,
                    info=ProduceConfig.model_fields['auto_set_memory'].description
                )
                auto_set_support = gr.Checkbox(
                    label="自动编成支援卡",
                    value=self.current_config.options.produce.auto_set_support_card,
                    info=ProduceConfig.model_fields['auto_set_support_card'].description
                )
                use_pt_boost = gr.Checkbox(
                    label="使用支援强化 Pt 提升",
                    value=self.current_config.options.produce.use_pt_boost,
                    info=ProduceConfig.model_fields['use_pt_boost'].description
                )
                use_note_boost = gr.Checkbox(
                    label="使用笔记数提升",
                    value=self.current_config.options.produce.use_note_boost,
                    info=ProduceConfig.model_fields['use_note_boost'].description
                )
                follow_producer = gr.Checkbox(
                    label="关注租借了支援卡的制作人",
                    value=self.current_config.options.produce.follow_producer,
                    info=ProduceConfig.model_fields['follow_producer'].description
                )
            
            produce_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[produce_enabled],
                outputs=[produce_group]
            )
        return produce_enabled, produce_mode, produce_count, produce_idols, auto_set_memory, auto_set_support, use_pt_boost, use_note_boost, follow_producer

    def _create_settings_tab(self) -> None:
        with gr.Tab("设置"):
            gr.Markdown("## 设置")
            
            # 模拟器设置
            with gr.Column():
                gr.Markdown("### 模拟器设置")
                adb_ip = gr.Textbox(
                    value=self.current_config.backend.adb_ip,
                    label="ADB IP 地址",
                    info=BackendConfig.model_fields['adb_ip'].description,
                    interactive=True
                )
                adb_port = gr.Number(
                    value=self.current_config.backend.adb_port,
                    label="ADB 端口",
                    info=BackendConfig.model_fields['adb_port'].description,
                    minimum=1,
                    maximum=65535,
                    step=1,
                    interactive=True
                )
                screenshot_impl = gr.Dropdown(
                    choices=['adb', 'adb_raw', 'uiautomator2'],
                    value=self.current_config.backend.screenshot_impl,
                    label="截图方法",
                    info=BackendConfig.model_fields['screenshot_impl'].description,
                    interactive=True
                )
                keep_screenshots = gr.Checkbox(
                    label="保留截图数据",
                    value=self.current_config.keep_screenshots,
                    info=UserConfig.model_fields['keep_screenshots'].description,
                    interactive=True
                )
            
            # 商店购买设置
            purchase_settings = self._create_purchase_settings()
            
            # 活动费设置
            with gr.Column():
                gr.Markdown("### 活动费设置")
                activity_funds = gr.Checkbox(
                    label="启用收取活动费",
                    value=self.current_config.options.activity_funds.enabled,
                    info=ActivityFundsConfig.model_fields['enabled'].description
                )
            
            # 礼物设置
            with gr.Column():
                gr.Markdown("### 礼物设置")
                presents = gr.Checkbox(
                    label="启用收取礼物",
                    value=self.current_config.options.presents.enabled,
                    info=PresentsConfig.model_fields['enabled'].description
                )
            
            # 工作设置
            work_settings = self._create_work_settings()
            
            # 竞赛设置
            with gr.Column():
                gr.Markdown("### 竞赛设置")
                contest = gr.Checkbox(
                    label="启用竞赛",
                    value=self.current_config.options.contest.enabled,
                    info=ContestConfig.model_fields['enabled'].description
                )
            
            # 培育设置
            produce_settings = self._create_produce_settings()
            
            # 任务奖励设置
            with gr.Column():
                gr.Markdown("### 任务奖励设置")
                mission_reward = gr.Checkbox(
                    label="启用领取任务奖励",
                    value=self.current_config.options.mission_reward.enabled,
                    info=MissionRewardConfig.model_fields['enabled'].description
                )
            
            save_btn = gr.Button("保存设置")
            result = gr.Markdown()
            
            # 收集所有设置组件
            all_settings = [
                adb_ip, adb_port, screenshot_impl, keep_screenshots,
                *purchase_settings,
                activity_funds,
                presents,
                *work_settings,
                contest,
                *produce_settings,
                mission_reward
            ]
            
            save_btn.click(
                fn=self.save_settings,
                inputs=all_settings,
                outputs=result
            )

    def _create_log_tab(self) -> None:
        with gr.Tab("日志"):
            gr.Markdown("## 日志")
            
            with gr.Column():
                with gr.Row():
                    export_dumps_btn = gr.Button("导出 dump")
                    export_logs_btn = gr.Button("导出日志")
                result_text = gr.Markdown("等待操作\n\n\n")
            
            export_dumps_btn.click(
                fn=self.export_dumps,
                outputs=[result_text]
            )
            export_logs_btn.click(
                fn=self.export_logs,
                outputs=[result_text]
            )

    def _load_config(self) -> None:
        # 加载配置文件
        config_path = "config.json"
        self.config = load_config(config_path, type=BaseConfig, use_default_if_not_found=True)
        if not self.config.user_configs:
            # 如果没有用户配置，创建一个默认配置
            default_config = UserConfig[BaseConfig](
                name="默认配置",
                category="default",
                description="默认配置",
                backend=BackendConfig(),
                options=BaseConfig()
            )
            self.config.user_configs.append(default_config)
        self.current_config = self.config.user_configs[0]

    def create_ui(self) -> gr.Blocks:
        with gr.Blocks(title="琴音小助手", css="#container { max-width: 800px; margin: auto; padding: 20px; }") as app:
            with gr.Column(elem_id="container"):
                gr.Markdown("# 琴音小助手")
                
                with gr.Tabs():
                    self._create_status_tab()
                    self._create_task_tab()
                    self._create_settings_tab()
                    self._create_log_tab()
            
        return app

def main() -> None:
    ui = KotoneBotUI()
    app = ui.create_ui()
    app.launch(inbrowser=True)

if __name__ == "__main__":
    main()



from kotonebot.tasks.common import sprite_path
from kotonebot.backend.core import Image, HintBox



class Common:
    
    ButtonClose = Image(path=sprite_path(r"e3b5cdcb-58b9-4090-b760-0c98b1615cf1.png"), name="button_close.png")

    ButtonCommuFastforward = Image(path=sprite_path(r"fdb3161a-db55-4bd4-8371-af55f3b0a847.png"), name="button_commu_fastforward.png")

    ButtonCommuSkip = Image(path=sprite_path(r"0a0ed135-3049-4762-9e40-7b709aee3bfd.png"), name="button_commu_skip.png")

    ButtonCompletion = Image(path=sprite_path(r"545f9608-a694-4498-bcad-cd57a365efa8.png"), name="button_completion.png")

    ButtonConfirm = Image(path=sprite_path(r"fd624f2c-8295-4dcd-a496-74eb37370b4d.png"), name="button_confirm.png")

    ButtonConfirmNoIcon = Image(path=sprite_path(r"9ecaa6ee-4986-4c3f-8590-060ecef96057.png"), name="button_confirm_no_icon.png")

    ButtonContest = Image(path=sprite_path(r"a77d19f7-5430-446c-9c23-0fcda7bee78e.png"), name="button_contest.png")

    ButtonEnd = Image(path=sprite_path(r"636f2c68-3215-4d35-9b3d-8bb5c7900604.png"), name="button_end.png")

    ButtonHome = Image(path=sprite_path(r"08b48ec8-3816-4d9e-8dcb-dfba8eb47519.png"), name="button_home.png")

    ButtonIconArrowShort = Image(path=sprite_path(r"7ba14e57-0e8d-449d-bb6b-0e363563cf54.png"), name="button_icon_arrow_short.png")

    ButtonIconArrowShortDisabled = Image(path=sprite_path(r"2cd66720-7afb-4e11-bcef-9c34573160b2.png"), name="button_icon_arrow_short_disabled.png")

    ButtonIconCheckMark = Image(path=sprite_path(r"afd291e2-2910-4f6d-bdb8-508d64c646f8.png"), name="button_icon_check_mark.png")

    ButtonIconClose = Image(path=sprite_path(r"685e8161-17d6-483c-8943-b43d6811079b.png"), name="button_icon_close.png")

    ButtonNext = Image(path=sprite_path(r"7f1f78fe-6a00-4c3a-9b16-bb9df1012eb1.png"), name="button_next.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"db611310-f91b-44c8-8dc1-682d7193adae.png"), name="button_next_no_icon.png")

    ButtonRetry = Image(path=sprite_path(r"79cfdc42-4978-4c67-8ced-79fafe7c9a66.png"), name="button_retry.png")

    ButtonSelect = Image(path=sprite_path(r"d8cea315-74f3-4ec9-a066-606c6974c145.png"), name="button_select.png")

    ButtonStart = Image(path=sprite_path(r"70b96ed0-3c8d-4321-92a6-a6a6f24b2223.png"), name="button_start.png")

    ButtonToolbarHome = Image(path=sprite_path(r"88d57dfb-4c4c-4d1a-b164-277843c9c301.png"), name="button_toolbar_home.png")

    CheckboxUnchecked = Image(path=sprite_path(r"e6a79ffc-acad-476c-b78b-6fcc433a8857.png"), name="checkbox_unchecked.png")

    ButtonToolbarHomeBinary = Image(path=sprite_path(r"2f68e457-4491-4ebc-b436-e3e63db92e14.png"), name="底部工具栏 首页按钮 二值化图片")

    ButtonToolbarMenuBinary = Image(path=sprite_path(r"bf6dcf21-2ffc-441f-82f3-59319f2313b1.png"), name="底部工具栏 菜单按钮 二值化图片")

    TextGameUpdate = Image(path=sprite_path(r"3b301b5a-0658-40ed-8193-f3f8d7d8ca2a.png"), name="text_game_update.png")

    TextNetworkError = Image(path=sprite_path(r"ff789005-f274-45ed-8dea-d9ce58dd0864.png"), name="text_network_error.png")


    pass
class Daily:
    
    ButonLinkData = Image(path=sprite_path(r"dccc72a8-7971-4836-8750-d0cd32cbc4f4.png"), name="buton_link_data.png")

    ButtonAssignmentPartial = Image(path=sprite_path(r"9b1d8702-7462-4a54-b213-5b15961dd6ac.png"), name="button_assignment_partial.png")

    ButtonClaimAllNoIcon = Image(path=sprite_path(r"d6a265c8-77d6-47ef-833f-ff06bf0c2d02.png"), name="button_claim_all_no_icon.png")

    ButtonContestChallenge = Image(path=sprite_path(r"290739f5-3994-476f-a1e5-e2452b5028e2.png"), name="button_contest_challenge.png")

    ButtonContestChallengeStart = Image(path=sprite_path(r"345a0c0b-8b3a-4ba1-983f-b28ed8459dee.png"), name="button_contest_challenge_start.png")

    ButtonContestRanking = Image(path=sprite_path(r"98b89394-b863-495b-8e89-2c3db99d7e7c.png"), name="button_contest_ranking.png")

    ButtonContestStart = Image(path=sprite_path(r"4b961dcf-1189-417c-945e-742e9cb8298a.png"), name="button_contest_start.png")

    ButtonDailyShop = Image(path=sprite_path(r"d9727aa1-50bc-4342-bf69-45294e6ad4dd.png"), name="button_daily_shop.png")

    ButtonHomeCurrent = Image(path=sprite_path(r"65b2fa7c-b4f3-4a89-a58f-b3d69d0ff492.png"), name="button_home_current.png")

    ButtonIconPass = Image(path=sprite_path(r"2b54178e-cebc-4172-a5cd-853ea4ca48b1.png"), name="button_icon_pass.png")

    ButtonIconSkip = Image(path=sprite_path(r"a4232f30-9bf1-4c12-a7aa-8cd68d92b050.png"), name="button_icon_skip.png")

    ButtonMission = Image(path=sprite_path(r"5508aa4a-94b0-44aa-bf7e-eed922835c3c.png"), name="button_mission.png")

    ButtonPass = Image(path=sprite_path(r"2238778e-981a-4a7d-8b93-b469f1dc0a9e.png"), name="button_pass.png")

    ButtonPassClaim = Image(path=sprite_path(r"d94177c8-78f6-4936-b0d7-5b038066d9fa.png"), name="button_pass_claim.png")

    ButtonPresentsPartial = Image(path=sprite_path(r"6d693394-cf66-4fb7-bb4a-711fe115f06d.png"), name="button_presents_partial.png")

    ButtonProduce = Image(path=sprite_path(r"942a0128-14ec-4ab4-9b2e-e1511a0ab207.png"), name="button_produce.png")

    ButtonShop = Image(path=sprite_path(r"beae8cfe-7320-40bf-85e9-53dc2b7e77c6.png"), name="button_shop.png")

    ButtonShopCountAdd = Image(path=sprite_path(r"cdf86260-9c8d-4389-8137-8583ad476d4b.png"), name="button_shop_count_add.png")

    ButtonShopCountAddDisabled = Image(path=sprite_path(r"e9281664-7c68-4963-9c28-1416ccc9be84.png"), name="button_shop_count_add_disabled.png")

    IconTitleDailyShop = Image(path=sprite_path(r"e9ee330d-dfca-440e-8b8c-0a3b4e8c8730.png"), name="日常商店标题图标")

    BoxHomeAssignment = HintBox(x1=16, y1=642, x2=127, y2=752, source_resolution=(720, 1280))

    BoxHomeAP = HintBox(x1=291, y1=4, x2=500, y2=82, source_resolution=(720, 1280))

    BoxHomeJewel = HintBox(x1=500, y1=7, x2=703, y2=82, source_resolution=(720, 1280))

    BoxHomeActivelyFunds = HintBox(x1=11, y1=517, x2=137, y2=637, source_resolution=(720, 1280))

    IconAssignKouchou = Image(path=sprite_path(r"2f9a0052-8613-4c9d-bbcd-97a0ee784224.png"), name="icon_assign_kouchou.png")

    IconAssignMiniLive = Image(path=sprite_path(r"e8185b4d-3f72-4736-9842-9ef4c06dc252.png"), name="icon_assign_mini_live.png")

    IconAssignOnlineLive = Image(path=sprite_path(r"0f4c13c0-8cec-401a-bcce-818218314211.png"), name="icon_assign_online_live.png")

    IconAssignTitle = Image(path=sprite_path(r"a384caea-3fbb-4d44-99f7-daffbc83c9ce.png"), name="icon_assign_title.png")

    IconShopAp = Image(path=sprite_path(r"4eb8735c-c8de-403f-94a5-fb14a4764e81.png"), name="icon_shop_ap.png")

    IconShopMoney = Image(path=sprite_path(r"bb1e4fc9-e16a-4694-a2b2-9f5ea2e4b35b.png"), name="icon_shop_money.png")

    IconShopTitle = Image(path=sprite_path(r"80e2682d-6a9f-42d1-8c0d-5fac5b82d17a.png"), name="icon_shop_title.png")

    IconTitleAssign = Image(path=sprite_path(r"6a2ddc90-1bb6-4fcb-91d4-e650e5c96254.png"), name="icon_title_assign.png")

    IconTitlePass = Image(path=sprite_path(r"c333d35e-46bb-4bdf-8685-0e2f06191dde.png"), name="icon_title_pass.png")

    ButtonAssignmentShortenTime = Image(path=sprite_path(r"1652f06a-5417-49ef-8949-4854772d9ab7.png"), name="工作页面 短缩 时间")

    TextActivityFundsMax = Image(path=sprite_path(r"09f4278a-b95d-40c9-a083-6072c45dc8e7.png"), name="text_activity_funds_max.png")

    TextAssignmentCompleted = Image(path=sprite_path(r"e533bb6b-8dce-4ed2-abcb-a454c3ded7e7.png"), name="text_assignment_completed.png")

    TextContest = Image(path=sprite_path(r"888fe566-7ec4-4158-9ffe-12beccc184a8.png"), name="text_contest.png")

    TextContestLastOngoing = Image(path=sprite_path(r"1ce41091-4235-46f0-a1ad-b6476a7e0b8e.png"), name="text_contest_last_ongoing.png")

    TextContestNoMemory = Image(path=sprite_path(r"22159eb3-c923-42e6-ada5-f6abeee38162.png"), name="text_contest_no_memory.png")

    TextContestOverallStats = Image(path=sprite_path(r"6848cd81-b7b6-4665-8efa-44be000d7892.png"), name="text_contest_overall_stats.png")

    TextShopPurchased = Image(path=sprite_path(r"13cd2e0a-9428-46c0-a184-1289cb2e094a.png"), name="text_shop_purchased.png")

    TextShopRecommended = Image(path=sprite_path(r"532f3468-64e2-4301-b189-951c78fc9ff2.png"), name="text_shop_recommended.png")

    TextTabShopAp = Image(path=sprite_path(r"e832e34d-eec6-4655-8df4-bc50c1a35888.png"), name="text_tab_shop_ap.png")


    pass
class Shop:
    
    ItemLessonNote = Image(path=sprite_path(r"0949c622-9067-4f0d-bac2-3f938a1d2ed2.png"), name="レッスンノート")

    ItemVeteranNote = Image(path=sprite_path(r"b2af59e9-60e3-4d97-8c72-c7ba092113a3.png"), name="ベテランノート")

    ItemSupportEnhancementPt = Image(path=sprite_path(r"835489e2-b29b-426c-b4c9-3bb9f8eb6195.png"), name="サポート強化Pt 支援强化Pt")

    ItemSenseNoteVocal = Image(path=sprite_path(r"c5b7d67e-7260-4f08-a0e9-4d31ce9bbecf.png"), name="センスノート（ボーカル）感性笔记（声乐）")

    ItemSenseNoteDance = Image(path=sprite_path(r"0f7d581d-cea3-4039-9205-732e4cd29293.png"), name="センスノート（ダンス）感性笔记（舞蹈）")

    ItemSenseNoteVisual = Image(path=sprite_path(r"d3cc3323-51af-4882-ae12-49e7384b746d.png"), name="センスノート（ビジュアル）感性笔记（形象）")

    ItemLogicNoteVocal = Image(path=sprite_path(r"a1df3af1-a3e7-4521-a085-e4dc3cd1cc57.png"), name="ロジックノート（ボーカル）理性笔记（声乐）")

    ItemLogicNoteDance = Image(path=sprite_path(r"a9fcaf04-0c1f-4b0f-bb5b-ede9da96180f.png"), name="ロジックノート（ダンス）理性笔记（舞蹈）")

    ItemLogicNoteVisual = Image(path=sprite_path(r"c3f536d6-a04a-4651-b3f9-dd2c22593f7f.png"), name="ロジックノート（ビジュアル）理性笔记（形象）")

    ItemAnomalyNoteVocal = Image(path=sprite_path(r"eef25cf9-afd0-43b1-b9c5-fbd997bd5fe0.png"), name="アノマリーノート（ボーカル）非凡笔记（声乐）")

    ItemAnomalyNoteDance = Image(path=sprite_path(r"df991b42-ed8e-4f2c-bf0c-aa7522f147b6.png"), name="アノマリーノート（ダンス）非凡笔记（舞蹈）")

    ItemAnomalyNoteVisual = Image(path=sprite_path(r"9340b854-025c-40da-9387-385d38433bef.png"), name="アノマリーノート（ビジュアル）非凡笔记（形象）")

    ItemRechallengeTicket = Image(path=sprite_path(r"ea1ba124-9cb3-4427-969a-bacd47e7d920.png"), name="再挑戦チケット 重新挑战券")

    ItemRecordKey = Image(path=sprite_path(r"1926f2f9-4bd7-48eb-9eba-28ec4efb0606.png"), name="記録の鍵  解锁交流的物品")

    class IdolPiece:
        
        倉本千奈_WonderScale = Image(path=sprite_path(r"6720b6e8-ae80-4cc0-a885-518efe12b707.png"), name="倉本千奈 WonderScale 碎片")
    
        篠泽广_光景 = Image(path=sprite_path(r"afa06fdc-a345-4384-b25d-b16540830256.png"), name="篠泽广 光景 碎片")
    
        紫云清夏_TameLieOneStep = Image(path=sprite_path(r"278b7d9c-707e-4392-9677-74574b5cdf42.png"), name="紫云清夏 Tame-Lie-One-Step 碎片")
    
    
        pass

    pass
class Produce:
    
    BoxProduceOngoing = HintBox(x1=179, y1=937, x2=551, y2=1091, source_resolution=(720, 1280))

    ButtonAutoSet = Image(path=sprite_path(r"d03ea7b6-da57-4a40-8e92-b4a4bbce0bb8.png"), name="button_auto_set.png")

    ButtonProduce = Image(path=sprite_path(r"c19640c8-3f23-4929-9def-1b6d7eb7c91a.png"), name="button_produce.png")

    ButtonProduceStart = Image(path=sprite_path(r"b8481e1f-75bb-452f-9b5a-d50e330a60a6.png"), name="button_produce_start.png")

    ButtonRegular = Image(path=sprite_path(r"7ad99675-e3ff-4fa6-9d29-9cabbfe3cf40.png"), name="button_regular.png")

    CheckboxIconNoteBoost = Image(path=sprite_path(r"a1a0a94f-7214-4a0b-9899-e6fc64e3a82c.png"), name="checkbox_icon_note_boost.png")

    CheckboxIconSupportPtBoost = Image(path=sprite_path(r"e3b62d4e-0bbf-44b4-b28a-1a4bff9d2b2f.png"), name="checkbox_icon_support_pt_boost.png")

    BoxModeButtons = HintBox(x1=7, y1=818, x2=713, y2=996, source_resolution=(720, 1280))

    ButtonPIdolOverview = Image(path=sprite_path(r"e88c9ad1-ec37-4fcd-b086-862e1e7ce8fd.png"), name="Pアイドルー覧  P偶像列表展示")

    BoxStepIndicator = HintBox(x1=4, y1=11, x2=405, y2=99, source_resolution=(720, 1280))

    IconPIdolLevel = Image(path=sprite_path(r"30a6f399-6999-4f04-bb77-651e0214112f.png"), name="P偶像卡上的等级图标")

    KbIdolOverviewName = HintBox(x1=140, y1=16, x2=615, y2=97, source_resolution=(720, 1280))

    ButtonResume = Image(path=sprite_path(r"ccbcb114-7f73-43d1-904a-3a7ae660c531.png"), name="再開する")

    BoxResumeDialogProduceType = HintBox(x1=186, y1=491, x2=541, y2=550, source_resolution=(720, 1280))

    BoxResumeDialogIdolTitle = HintBox(x1=194, y1=794, x2=523, y2=840, source_resolution=(720, 1280))

    RadioTextSkipCommu = Image(path=sprite_path(r"ab56463b-a268-4b59-89cf-c180160204f0.png"), name="radio_text_skip_commu.png")

    TextAPInsufficient = Image(path=sprite_path(r"4883c564-f950-4a29-9f5f-6f924123cd22.png"), name="培育 AP 不足提示弹窗 标题")

    ButtonRefillAP = Image(path=sprite_path(r"eaba6ebe-f0df-4918-aee5-ef4e3ffedcf0.png"), name="确认恢复AP按钮")

    ScreenshotNoEnoughAp2 = Image(path=sprite_path(r"cc673f49-eee6-47af-ae8c-e99726776ec0.png"), name="screenshot_no_enough_ap_2.png")

    TextRentAvailable = Image(path=sprite_path(r"af561656-5214-4d0d-aa9b-eb57e275b224.png"), name="text_rent_available.png")


    pass
class InPurodyuusu:
    
    A = Image(path=sprite_path(r"b9c5e83e-4f5f-4813-9b82-932867065383.png"), name="A.png")

    AcquireBtnDisabled = Image(path=sprite_path(r"e65fd66c-0fc3-4090-8d55-af50b753c0d0.png"), name="acquire_btn_disabled.png")

    ButtonCancel = Image(path=sprite_path(r"74fc7bf9-0079-4bbe-a264-7a6077de698a.png"), name="button_cancel.png")

    ButtonComplete = Image(path=sprite_path(r"f6b5bc5b-ec13-469b-87c5-80601b5e475d.png"), name="button_complete.png")

    ButtonFinalPracticeDance = Image(path=sprite_path(r"68def82e-472e-4fe9-9148-2d68324ee7eb.png"), name="button_final_practice_dance.png")

    ButtonFinalPracticeVisual = Image(path=sprite_path(r"04f9aed6-d542-4a11-999d-77adb45e4cc2.png"), name="button_final_practice_visual.png")

    ButtonFinalPracticeVocal = Image(path=sprite_path(r"e0fdc58d-8602-4649-ad3b-3ab0941e09e4.png"), name="button_final_practice_vocal.png")

    ButtonFollowNoIcon = Image(path=sprite_path(r"7dfbb44e-a169-40be-8f16-c719e9b66b7d.png"), name="button_follow_no_icon.png")

    ButtonIconStudy = Image(path=sprite_path(r"302dcb9b-ceb1-4df1-87d5-0059b2f3360c.png"), name="button_icon_study.png")

    ButtonIconStudyVisual = Image(path=sprite_path(r"963c4066-ba01-4e6e-9818-91835eaf1711.png"), name="button_icon_study_visual.png")

    ButtonLeave = Image(path=sprite_path(r"dc2a50fe-e0aa-4972-901a-ba73290b592f.png"), name="button_leave.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"4c1de807-85ea-45e6-86be-509567dac54e.png"), name="button_next_no_icon.png")

    ButtonNotAcquire = Image(path=sprite_path(r"9e63b633-a7c2-4c93-8612-b5edfee49f2d.png"), name="button_not_acquire.png")

    ButtonRetry = Image(path=sprite_path(r"4eaa3665-d81a-4e75-aa1e-8b3523649e65.png"), name="button_retry.png")

    ButtonTextActionOuting = Image(path=sprite_path(r"c7bfe369-8a93-4006-a362-a4dd8dcc9b7c.png"), name="button_text_action_outing.png")

    ButtonTextAllowance = Image(path=sprite_path(r"60f98b5d-cfe5-459c-a9c2-8303a89cb70c.png"), name="button_text_allowance.png")

    ButtonTextConsult = Image(path=sprite_path(r"7bf6e99b-333d-4b40-b7b8-28d053aa592d.png"), name="button_text_consult.png")

    IconClearBlue = Image(path=sprite_path(r"77180382-5c9b-4d38-9477-9a42775abf8f.png"), name="icon_clear_blue.png")

    IconTitleAllowance = Image(path=sprite_path(r"3f62e201-8a35-43dd-b182-a9c6d7c39a37.png"), name="icon_title_allowance.png")

    IconTitleStudy = Image(path=sprite_path(r"f04e6279-a2ad-44fa-a346-f2f132cd3e6b.png"), name="icon_title_study.png")

    LootboxSliverLock = Image(path=sprite_path(r"30bbd6d3-eb46-43b3-9434-606d4ff4a642.png"), name="lootbox_sliver_lock.png")

    LootBoxSkillCard = Image(path=sprite_path(r"347ec27e-8f6a-4b28-a08f-8455d5a371ad.png"), name="loot_box_skill_card.png")

    M = Image(path=sprite_path(r"543b5354-002d-422c-8346-7b70bdbd4bda.png"), name="M.png")

    BoxWeeksUntilExam = HintBox(x1=11, y1=8, x2=237, y2=196, source_resolution=(720, 1280))

    BoxExamTop = HintBox(x1=5, y1=2, x2=712, y2=55, source_resolution=(720, 1280))

    PDrinkIcon = Image(path=sprite_path(r"c93b7555-2445-4452-9725-76ba274cfe93.png"), name="p_drink_icon.png")

    PItemIconColorful = Image(path=sprite_path(r"e5699f8d-d853-491f-9565-67219dfcc58e.png"), name="p_item_icon_colorful.png")

    PSkillCardIconBlue = Image(path=sprite_path(r"7bdb8a9f-0dbc-4c92-aceb-24c74c46eeb3.png"), name="p_skill_card_icon_blue.png")

    PSkillCardIconColorful = Image(path=sprite_path(r"cf38fe4e-5ae8-44c2-a03d-bc81a689a270.png"), name="p_skill_card_icon_colorful.png")

    Rest = Image(path=sprite_path(r"6bc6cd1c-096b-49fd-bec7-c5197c5d6f75.png"), name="rest.png")

    RestConfirmBtn = Image(path=sprite_path(r"a1f0f268-2b46-4828-82cd-96b4d1b038ec.png"), name="rest_confirm_btn.png")

    BoxNoSkillCard = HintBox(x1=180, y1=977, x2=529, y2=1026, source_resolution=(720, 1280))

    BoxPDrinkMaxConfirmTitle = HintBox(x1=46, y1=829, x2=270, y2=876, source_resolution=(720, 1280))

    BoxSelectPStuff = HintBox(x1=62, y1=558, x2=655, y2=716, source_resolution=(720, 1280))

    BoxSelectPStuffComfirm = HintBox(x1=256, y1=1064, x2=478, y2=1128, source_resolution=(720, 1280))

    BoxSkillCardAcquired = HintBox(x1=194, y1=712, x2=528, y2=765, source_resolution=(720, 1280))

    BoxSkillCardEnhaced = HintBox(x1=49, y1=948, x2=676, y2=1106, source_resolution=(720, 1280))

    BoxCommuEventButtonsArea = HintBox(x1=14, y1=412, x2=703, y2=1089, source_resolution=(720, 1280))

    TextAsariProduceEnd = Image(path=sprite_path(r"55972f3f-d576-45b6-9c4e-2ad8d46200ef.png"), name="text_asari_produce_end.png")

    TextButtonExamSkipTurn = Image(path=sprite_path(r"214efc0e-70ac-418f-a6ed-bcc793e108f9.png"), name="text_button_exam_skip_turn.png")

    TextClearUntil = Image(path=sprite_path(r"9b3e0f9f-0cd5-47db-89f2-9621d3dc66cf.png"), name="text_clear_until.png")

    TextDance = Image(path=sprite_path(r"06c22a4a-3d77-479a-8e75-f9ce12f2d927.png"), name="text_dance.png")

    TextFinalProduceRating = Image(path=sprite_path(r"fabc1f1e-a6c5-4b6d-b895-ca7a7968ef6e.png"), name="text_final_produce_rating.png")

    TextOneWeekRemaining = Image(path=sprite_path(r"c00ef56a-b994-4766-b99d-4e7a88f6c88c.png"), name="text_one_week_remaining.png")

    TextPerfectUntil = Image(path=sprite_path(r"f6272354-1e9e-4d4e-b679-78943bf3f4da.png"), name="text_perfect_until.png")

    TextPleaseSelectPDrink = Image(path=sprite_path(r"04f987f0-4056-48ef-ba72-9cc596b81369.png"), name="text_please_select_p_drink.png")

    TextPDiary = Image(path=sprite_path(r"573a4bbe-9458-44ba-8c35-d0e36100a9e1.png"), name="text_p_diary.png")

    TextPDrinkMax = Image(path=sprite_path(r"d87c8434-24a1-4ec6-95bf-ec74403d7708.png"), name="text_p_drink_max.png")

    TextSenseiTipConsult = Image(path=sprite_path(r"635c6e8e-7079-42fe-b9a0-2c3b4b48c076.png"), name="text_sensei_tip_consult.png")

    TextSenseiTipDance = Image(path=sprite_path(r"775451ff-63a5-4d1b-b47c-842d615ec266.png"), name="text_sensei_tip_dance.png")

    TextSenseiTipRest = Image(path=sprite_path(r"a779eb78-6aec-47e9-85e1-9dff881644b9.png"), name="text_sensei_tip_rest.png")

    TextSenseiTipVisual = Image(path=sprite_path(r"96adda6c-19e7-4109-b541-e84ca7df8562.png"), name="text_sensei_tip_visual.png")

    TextSenseiTipVocal = Image(path=sprite_path(r"8b35d64f-34e3-4969-9552-b7da0644dbef.png"), name="text_sensei_tip_vocal.png")

    TextSkipTurnDialog = Image(path=sprite_path(r"dd9b55bc-dc79-4f2f-a53a-a6f796711d65.png"), name="text_skip_turn_dialog.png")

    TextVisual = Image(path=sprite_path(r"80effacb-5f16-4fcf-a151-581f97475393.png"), name="text_visual.png")

    class Action:
        
        ActionStudy = Image(path=sprite_path(r"44f8fb09-1b51-4178-9180-6bd34ae534e1.png"), name="action_study.png")
    
        PDorinkuBg = Image(path=sprite_path(r"0eb0ca65-9c2d-4eb6-9b30-9dd423c46204.png"), name="p_dorinku_bg.png")
    
        PDorinkuBgMask = Image(path=sprite_path(r"ca629da2-0655-4e68-9bf0-5eb40a64834d.png"), name="p_dorinku_bg_mask.png")
    
        PItem = Image(path=sprite_path(r"fd0c8246-fb6b-48a7-bf98-49954458bc81.png"), name="p_item.png")
    
        VocalWhiteBg = Image(path=sprite_path(r"d664f6b6-62fe-4542-9ac5-23ea006b9eb9.png"), name="vocal_white_bg.png")
    
    
        pass

    pass


from logging import getLogger

from .. import R
from kotonebot import device, ocr, image, Rect, action, sleep, contains

logger = getLogger(__name__)

# TODO: 加入一个 device.snapshot() 方法，用于保存当前设备画面，避免重复截图。
# TODO: 比较 OCR 和模板匹配文字的性能，如果模板匹配更好，
# 则首次使用 OCR，找到结果后自动截图，后续使用模板匹配。

@action('检测是否在 P 饮料领取中')
def is_on_pdorinku_acquisition() -> bool:
    """检查是否在 P 饮料领取中"""
    return ocr.find('受け取るＰドリンクを選んでください。') is not None

@action('列出当前要领取的 P 饮料')
def list_pdorinku() -> list[tuple[str, Rect]]:
    """
    列出当前要领取的 P 饮料

    :return: 检测结果。`[(饮料名称, 饮料矩形坐标), ...]`
    """
    # 截图所有饮料
    # TODO: 自动记录未知饮料
    dorinkus = image.find_all_crop(
        R.InPurodyuusu.Action.PDorinkuBg,
        mask=R.InPurodyuusu.Action.PDorinkuBgMask,
    )
    return [
        ('', dorinku.rect) # TODO: 获取饮料名称
        for dorinku in dorinkus
    ]

@action('领取 P 饮料')
def acquire_pdorinku(index: int):
    """
    领取 P 饮料
    :param index: 要领取的 P 饮料的索引。从 0 开始。
    """
    # TODO: 随机领取一个饮料改成根据具体情况确定最佳
    # 如果能不领取，就不领取
    if ocr.find(contains('受け取らない')):
        device.click()
        sleep(0.5)
        device.click(image.expect(R.InPurodyuusu.ButtonNotAcquire))
        sleep(0.5)
        device.click(image.expect(R.Common.ButtonConfirm))
    else:
        # 点击饮料
        drinks = list_pdorinku()
        dorinku = drinks[index]
        device.click(dorinku[1])
        logger.debug(f"Pドリンク clicked: {dorinku[0]}")
        sleep(0.3)
        # 确定按钮
        ret = ocr.expect('受け取る')
        device.click(ret.rect)
        logger.debug("受け取る clicked")
        sleep(1.3)
        # 再次确定
        device.click_center()
        logger.debug("再次确定 clicked")


__actions__ = [acquire_pdorinku]

if __name__ == '__main__':
    from pprint import pprint as print
    # print(list_pdorinku())
    acquire_pdorinku(0)
    input()



import time
from typing import Literal

import numpy as np
import uiautomator2 as u2
from cv2.typing import MatLike

from kotonebot import logging
from ..device import Device
from ..protocol import Screenshotable, Commandable, Touchable

logger = logging.getLogger(__name__)

SCREENSHOT_INTERVAL = 0.2

class UiAutomator2Impl(Screenshotable, Commandable, Touchable):
    def __init__(self, device: Device):
        self.device = device
        self.u2_client = u2.Device(device.adb.serial)
        self.__last_screenshot_time = 0
        
    def screenshot(self) -> MatLike:
        """
        截图
        """
        from kotonebot import sleep
        delta = time.time() - self.__last_screenshot_time
        if delta < SCREENSHOT_INTERVAL:
            time.sleep(SCREENSHOT_INTERVAL - delta)
        start_time = time.time()
        image = self.u2_client.screenshot(format='opencv')
        logger.verbose(f'uiautomator2 screenshot: {time.time() - start_time}s')
        self.__last_screenshot_time = time.time()
        assert isinstance(image, np.ndarray)
        return image
    
    @property
    def screen_size(self) -> tuple[int, int]:
        info = self.u2_client.info
        sizes = info['displayWidth'], info['displayHeight']
        if self.device.orientation == 'landscape':
            return (max(sizes), min(sizes))
        else:
            return (min(sizes), max(sizes))
    
    def detect_orientation(self) -> Literal['portrait', 'landscape'] | None:
        """
        检测设备方向
        """
        orientation = self.u2_client.info['displayRotation']
        if orientation == 1:
            return 'portrait'
        elif orientation == 0:
            return 'landscape'
        else:
            return None
    
    def launch_app(self, package_name: str) -> None:
        """
        启动应用
        """
        self.u2_client.app_start(package_name)
        
    def current_package(self) -> str | None:
        """
        获取当前应用包名
        """
        try:
            result = self.u2_client.app_current()
            logger.verbose(f'uiautomator2 current_package: {result}')
            return result['package']
        except:
            return None
        
    def click(self, x: int, y: int) -> None:
        """
        点击屏幕
        """
        self.u2_client.click(x, y)
        
    def swipe(self, x1: int, y1: int, x2: int, y2: int, duration: float|None = None) -> None:
        """
        滑动屏幕
        """
        self.u2_client.swipe(x1, y1, x2, y2, duration=duration or 0.1)



import os
import tempfile
from pathlib import Path
from typing import Sequence

import cv2
import requests
import numpy as np
from cv2.typing import MatLike
from dotenv import load_dotenv

load_dotenv()

def _save_temp_image(image: MatLike) -> Path:
    """将OpenCV图片保存为临时文件"""
    temp_file = Path(tempfile.mktemp(suffix='.jpg'))
    cv2.imwrite(str(temp_file), image)
    return temp_file

def _upload_single(image: MatLike | str) -> str:
    """
    上传单张图片到freeimage.host
    
    :param image: OpenCV MatLike 或本地图片文件路径
    """
    api_url = 'https://freeimage.host/api/1/upload'
    api_key = os.getenv('FREEIMAGEHOST_KEY')
    
    if not api_key:
        raise RuntimeError('Environment variable FREEIMAGEHOST_KEY is not set')
    
    # 处理输入
    temp_file = None
    if isinstance(image, str):
        # 本地文件路径
        files = {'source': open(image, 'rb')}
    else:
        # 保存OpenCV图片为临时文件
        temp_file = _save_temp_image(image)
        files = {'source': open(temp_file, 'rb')}
    
    data = {
        'key': api_key,
        'action': 'upload',
        'format': 'json'
    }
    
    try:
        # 发送POST请求
        response = requests.post(api_url, data=data, files=files)
        
        if response.status_code != 200:
            raise RuntimeError(f'Upload failed: HTTP {response.status_code}')
            
        result = response.json()
        
        if result['status_code'] != 200:
            raise RuntimeError(f'Upload failed: API {result["status_txt"]}')
            
        return result['image']['url']
        
    finally:
        # 清理临时文件
        files['source'].close()
        if temp_file and temp_file.exists():
            temp_file.unlink()

def upload(images: MatLike | str | Sequence[MatLike | str]) -> list[str]:
    """上传一张或多张图片到freeimage.host
    
    Args:
        images: 单张图片或图片列表。每个图片可以是OpenCV图片对象或本地图片文件路径
        
    Returns:
        上传后的图片URL列表
    """
    if isinstance(images, (str, np.ndarray)):
        _images = [images]
    elif isinstance(images, Sequence):
        _images = [img for img in images]
    else:
        raise ValueError("Invalid input type")
        
    return [_upload_single(img) for img in _images]

if __name__ == "__main__":
    print(upload(cv2.imread("res/sprites/jp/common/button_close.png")))



import os
import runpy
import shutil
import argparse
import importlib
from pathlib import Path
from threading import Thread

from . import debug
from kotonebot import logging
from kotonebot.backend.context import init_context

logger = logging.getLogger(__name__)

def _task_thread(task_module: str):
    """任务线程。"""
    runpy.run_module(task_module, run_name="__main__")

def _parse_args():
    """解析命令行参数。"""
    parser = argparse.ArgumentParser(description='KotoneBot visual debug tool')
    parser.add_argument(
        '-s', '--save', 
        help='Save dump image and results to the specified folder',
        type=str,
        metavar='PATH'
    )
    parser.add_argument(
        '-c', '--clear',
        help='Clear the dump folder before running',
        action='store_true'
    )
    parser.add_argument(
        '-t', '--config-type',
        help='The full path of the config data type. e.g. `kotonebot.tasks.common.BaseConfig`',
        type=str,
        metavar='TYPE',
        required=True
    )
    parser.add_argument(
        'input_module',
        help='The module to run'
    )
    return parser.parse_args()

def _start_task_thread(module: str):
    """启动任务线程。"""
    thread = Thread(target=_task_thread, args=(module,))
    thread.start()

if __name__ == "__main__":
    args = _parse_args()
    debug.enabled = True
    
    # 设置保存路径
    if args.save:
        save_path = Path(args.save)
        debug.auto_save_to_folder = str(save_path)
        if not os.path.exists(save_path):
            os.makedirs(save_path)
    if args.clear:
        if debug.auto_save_to_folder:
            try:
                logger.info(f"Removing {debug.auto_save_to_folder}")
                shutil.rmtree(debug.auto_save_to_folder)
            except PermissionError:
                logger.warning(f"Failed to remove {debug.auto_save_to_folder}. Trying to remove all contents instead.")
                for root, dirs, files in os.walk(debug.auto_save_to_folder):
                    for file in files:
                        try:
                            os.remove(os.path.join(root, file))
                        except PermissionError:
                            raise
                

    # 初始化上下文
    module_name, class_name = args.config_type.rsplit('.', 1)
    class_ = importlib.import_module(module_name).__getattribute__(class_name)
    init_context(config_type=class_)
    
    # 启动服务器
    from .server import app
    import uvicorn
    
    # 启动任务线程
    _start_task_thread(args.input_module)
    
    # 启动服务器
    uvicorn.run(app, host="127.0.0.1", port=8000, log_level='critical' if debug.hide_server_log else None)



"""
此文件包含非练习周的行动。

具体包括：おでかけ、相談、活動支給、授業
"""
from logging import getLogger

from kotonebot.backend.dispatch import SimpleDispatcher
from kotonebot.backend.util import Interval

from .. import R
from ..game_ui import CommuEventButtonUI, EventButton
from .common import acquisitions, AcquisitionType
from kotonebot import device, image, ocr, debug, action, sleep
from kotonebot.errors import UnrecoverableError
from ..actions.loading import wait_loading_end, wait_loading_start

logger = getLogger(__name__)

@action('检测是否可以执行活動支給')
def allowance_available():
    """
    判断是否可以执行活動支給。
    """
    return image.find(R.InPurodyuusu.ButtonTextAllowance) is not None

@action('检测是否可以执行授業')
def study_available():
    """
    判断是否可以执行授業。
    """
    # [screenshots/produce/action_study1.png]
    return image.find(R.InPurodyuusu.ButtonIconStudy) is not None

@action('执行授業')
def enter_study():
    """
    执行授業。

    前置条件：位于行动页面，且所有行动按钮清晰可见 \n
    结束状态：选择选项后可能会出现的，比如领取奖励、加载画面等。
    """
    logger.info("Executing 授業.")
    # [screenshots/produce/action_study1.png]
    logger.debug("Double clicking on 授業.")
    device.double_click(image.expect_wait(R.InPurodyuusu.ButtonIconStudy))
    # 等待进入页面。中间可能会出现未读交流
    # [screenshots/produce/action_study2.png]
    while not image.find(R.InPurodyuusu.IconTitleStudy):
        logger.debug("Waiting for 授業 screen.")
        acquisitions()
    # 获取三个选项的内容
    ui = CommuEventButtonUI()
    buttons = ui.all()
    if not buttons:
        raise UnrecoverableError("Failed to find any buttons.")
    # 选中 +30 的选项
    target_btn = next((btn for btn in buttons if '+30' in btn.description), None)
    if target_btn is None:
        logger.error("Failed to find +30 option. Pick the first button instead.")
        target_btn = buttons[0]
    logger.debug('Clicking "%s".', target_btn.description)
    if target_btn.selected:
        device.click(target_btn)
    else:
        device.double_click(target_btn)
    while acquisitions() is None:
        logger.info("Waiting for acquisitions finished.")
    logger.info("授業 completed.")


@action('执行活動支給')
def enter_allowance():
    """
    执行活動支給。
    
    前置条件：位于行动页面，且所有行动按钮清晰可见 \n
    结束状态：位于行动页面
    """
    logger.info("Executing 活動支給.")
    # 点击活動支給 [screenshots\allowance\step_1.png]
    logger.info("Double clicking on 活動支給.")
    device.double_click(image.expect(R.InPurodyuusu.ButtonTextAllowance), interval=1)
    # 等待进入页面
    while not image.find(R.InPurodyuusu.IconTitleAllowance):
        logger.debug("Waiting for 活動支給 screen.")
        acquisitions()
    # 领取奖励
    it = Interval()
    while True:
        # TODO: 检测是否在行动页面应当单独一个函数
        if image.find_multi([
            R.InPurodyuusu.TextPDiary, # 普通周
            R.InPurodyuusu.ButtonFinalPracticeDance # 离考试剩余一周
        ]):
            break
        if image.find(R.InPurodyuusu.LootboxSliverLock):
            logger.info("Click on lootbox.")
            device.click()
            sleep(0.5) # 防止点击了第一个箱子后立马点击了第二个
            continue
        if acquisitions() is not None:
            continue
        it.wait()
    logger.info("活動支給 completed.")

@action('判断是否可以休息')
def is_rest_available():
    """
    判断是否可以休息。
    """
    return image.find(R.InPurodyuusu.Rest) is not None


@action('执行休息')
def rest():
    """执行休息"""
    logger.info("Rest for this week.")
    (SimpleDispatcher('in_produce.rest')
        # 点击休息
        .click(R.InPurodyuusu.Rest)
        # 确定
        .click(R.InPurodyuusu.RestConfirmBtn, finish=True)
    ).run()

if __name__ == '__main__':
    from kotonebot.backend.context import manual_context, init_context
    init_context()
    manual_context().begin()
    # 获取三个选项的内容
    ui = CommuEventButtonUI()
    buttons = ui.all()
    if not buttons:
        raise UnrecoverableError("Failed to find any buttons.")
    # 选中 +30 的选项
    target_btn = next((btn for btn in buttons if btn.description == '+30'), None)
    if target_btn is None:
        logger.error("Failed to find +30 option. Pick the first button instead.")
        target_btn = buttons[0]
    # 固定点击 Vi. 选项
    logger.debug('Clicking "%s".', target_btn.description)
    if target_btn.selected:
        device.click(target_btn)
    else:
        device.double_click(target_btn)
    while acquisitions() is None:
        logger.info("Waiting for acquisitions finished.")
    logger.info("授業 completed.")



from enum import Enum
from typing import Literal

from .device import Device
from .implements.adb import AdbImpl
from .implements.adb_raw import AdbRawImpl
from .implements.uiautomator2 import UiAutomator2Impl

from adbutils import adb

DeviceImpl = Literal['adb', 'adb_raw', 'uiautomator2']

def create_device(
    addr: str,
    impl: DeviceImpl,
) -> Device:
    adb.connect(addr)
    d = [d for d in adb.device_list() if d.serial == addr]
    if len(d) == 0:
        raise ValueError(f"Device {addr} not found")
    d = d[0]
    device = Device(d)
    if impl == 'adb':
        device._command = AdbImpl(device)
        device._touch = AdbImpl(device)
        device._screenshot = AdbImpl(device)
    elif impl == 'adb_raw':
        device._command = AdbRawImpl(device)
        device._touch = AdbRawImpl(device)
        device._screenshot = AdbRawImpl(device)
    elif impl == 'uiautomator2':
        device._command = UiAutomator2Impl(device)
        device._touch = UiAutomator2Impl(device)
        device._screenshot = UiAutomator2Impl(device)
    return device



from importlib import resources
import os
from typing import Literal, Dict
from enum import IntEnum, Enum

from pydantic import BaseModel, ConfigDict

# TODO: from kotonebot import config (context) 会和 kotonebot.config 冲突
from kotonebot.backend.context import config

class Priority(IntEnum):
    START_GAME = 1
    DEFAULT = 0
    CLAIM_MISSION_REWARD = -1


class APShopItems(IntEnum):
    PRODUCE_PT_UP = 0
    """获取支援强化 Pt 提升"""
    PRODUCE_NOTE_UP = 1
    """获取笔记数提升"""
    RECHALLENGE = 2
    """再挑战券"""
    REGENERATE_MEMORY = 3
    """回忆再生成券"""


class PIdol(Enum):
    """P偶像"""
    倉本千奈_Campusmode = ["倉本千奈", "Campus mode!!"]
    倉本千奈_WonderScale = ["倉本千奈", "Wonder Scale"]
    倉本千奈_ようこそ初星温泉 = ["倉本千奈", "ようこそ初星温泉"]
    倉本千奈_仮装狂騒曲 = ["倉本千奈", "仮装狂騒曲"]
    倉本千奈_初心 = ["倉本千奈", "初心"]
    倉本千奈_学園生活 = ["倉本千奈", "学園生活"]
    倉本千奈_日々_発見的ステップ = ["倉本千奈", "日々、発見的ステップ！"]
    倉本千奈_胸を張って一歩ずつ = ["倉本千奈", "胸を張って一歩ずつ"]
    十王星南_Campusmode = ["十王星南", "Campus mode!!"]
    十王星南_一番星 = ["十王星南", "一番星"]
    十王星南_学園生活 = ["十王星南", "学園生活"]
    十王星南_小さな野望 = ["十王星南", "小さな野望"]
    姫崎莉波_clumsytrick = ["姫崎莉波", "clumsy trick"]
    姫崎莉波_私らしさのはじまり = ["姫崎莉波", "『私らしさ』のはじまり"]
    姫崎莉波_キミとセミブルー = ["姫崎莉波", "キミとセミブルー"]
    姫崎莉波_Campusmode = ["姫崎莉波", "Campus mode!!"]
    姫崎莉波_LUV = ["姫崎莉波", "L.U.V"]
    姫崎莉波_ようこそ初星温泉 = ["姫崎莉波", "ようこそ初星温泉"]
    姫崎莉波_ハッピーミルフィーユ = ["姫崎莉波", "ハッピーミルフィーユ"]
    姫崎莉波_初心 = ["姫崎莉波", "初心"]
    姫崎莉波_学園生活 = ["姫崎莉波", "学園生活"]
    月村手毬_Lunasaymaybe = ["月村手毬", "Luna say maybe"]
    月村手毬_一匹狼 = ["月村手毬", "一匹狼"]
    月村手毬_Campusmode = ["月村手毬", "Campus mode!!"]
    月村手毬_アイヴイ = ["月村手毬", "アイヴイ"]
    月村手毬_初声 = ["月村手毬", "初声"]
    月村手毬_学園生活 = ["月村手毬", "学園生活"]
    月村手毬_仮装狂騒曲 = ["月村手毬", "仮装狂騒曲"]
    有村麻央_Fluorite = ["有村麻央", "Fluorite"]
    有村麻央_はじまりはカッコよく = ["有村麻央", "はじまりはカッコよく"]
    有村麻央_Campusmode = ["有村麻央", "Campus mode!!"]
    有村麻央_FeelJewelDream = ["有村麻央", "Feel Jewel Dream"]
    有村麻央_キミとセミブルー = ["有村麻央", "キミとセミブルー"]
    有村麻央_初恋 = ["有村麻央", "初恋"]
    有村麻央_学園生活 = ["有村麻央", "学園生活"]
    篠泽广_コントラスト = ["篠泽广", "コントラスト"]
    篠泽广_一番向いていないこと = ["篠泽广", "一番向いていないこと"]
    篠泽广_光景 = ["篠泽广", "光景"]
    篠泽广_Campusmode = ["篠泽广", "Campus mode!!"]
    篠泽广_仮装狂騒曲 = ["篠泽广", "仮装狂騒曲"]
    篠泽广_ハッピーミルフィーユ = ["篠泽广", "ハッピーミルフィーユ"]
    篠泽广_初恋 = ["篠泽广", "初恋"]
    篠泽广_学園生活 = ["篠泽广", "学園生活"]
    紫云清夏_TameLieOneStep = ["紫云清夏", "Tame-Lie-One-Step"]
    紫云清夏_カクシタワタシ = ["紫云清夏", "カクシタワタシ"]
    紫云清夏_夢へのリスタート = ["紫云清夏", "夢へのリスタート"]
    紫云清夏_Campusmode = ["紫云清夏", "Campus mode!!"]
    紫云清夏_キミとセミブルー = ["紫云清夏", "キミとセミブルー"]
    紫云清夏_初恋 = ["紫云清夏", "初恋"]
    紫云清夏_学園生活 = ["紫云清夏", "学園生活"]
    花海佑芽_WhiteNightWhiteWish = ["花海佑芽", "White Night! White Wish!"]
    花海佑芽_学園生活 = ["花海佑芽", "学園生活"]
    花海佑芽_Campusmode = ["花海佑芽", "Campus mode!!"]
    花海佑芽_TheRollingRiceball = ["花海佑芽", "The Rolling Riceball"]
    花海佑芽_アイドル_はじめっ = ["花海佑芽", "アイドル、はじめっ！"]
    花海咲季_BoomBoomPow = ["花海咲季", "Boom Boom Pow"]
    花海咲季_Campusmode = ["花海咲季", "Campus mode!!"]
    花海咲季_FightingMyWay = ["花海咲季", "Fighting My Way"]
    花海咲季_わたしが一番 = ["花海咲季", "わたしが一番！"]
    花海咲季_冠菊 = ["花海咲季", "冠菊"]
    花海咲季_初声 = ["花海咲季", "初声"]
    花海咲季_古今東西ちょちょいのちょい = ["花海咲季", "古今東西ちょちょいのちょい"]
    花海咲季_学園生活 = ["花海咲季", "学園生活"]
    葛城リーリヤ_一つ踏み出した先に = ["葛城リーリヤ", "一つ踏み出した先に"]
    葛城リーリヤ_白線 = ["葛城リーリヤ", "白線"]
    葛城リーリヤ_Campusmode = ["葛城リーリヤ", "Campus mode!!"]
    葛城リーリヤ_WhiteNightWhiteWish = ["葛城リーリヤ", "White Night! White Wish!"]
    葛城リーリヤ_冠菊 = ["葛城リーリヤ", "冠菊"]
    葛城リーリヤ_初心 = ["葛城リーリヤ", "初心"]
    葛城リーリヤ_学園生活 = ["葛城リーリヤ", "学園生活"]
    藤田ことね_カワイイ_はじめました = ["藤田ことね", "カワイイ", "はじめました"]
    藤田ことね_世界一可愛い私 = ["藤田ことね", "世界一可愛い私"]
    藤田ことね_Campusmode = ["藤田ことね", "Campus mode!!"]
    藤田ことね_YellowBigBang = ["藤田ことね", "Yellow Big Bang！"]
    藤田ことね_WhiteNightWhiteWish = ["藤田ことね", "White Night! White Wish!"]
    藤田ことね_冠菊 = ["藤田ことね", "冠菊"]
    藤田ことね_初声 = ["藤田ことね", "初声"]
    藤田ことね_学園生活 = ["藤田ことね", "学園生活"]

class DailyMoneyShopItems(IntEnum):
    """日常商店物品"""
    Recommendations = -1
    """所有推荐商品"""
    LessonNote = 0
    """レッスンノート"""
    VeteranNote = 1
    """ベテランノート"""
    SupportEnhancementPt = 2
    """サポート強化Pt 支援强化Pt"""
    SenseNoteVocal = 3
    """センスノート（ボーカル）感性笔记（声乐）"""
    SenseNoteDance = 4
    """センスノート（ダンス）感性笔记（舞蹈）"""
    SenseNoteVisual = 5
    """センスノート（ビジュアル）感性笔记（形象）"""
    LogicNoteVocal = 6
    """ロジックノート（ボーカル）理性笔记（声乐）"""
    LogicNoteDance = 7
    """ロジックノート（ダンス）理性笔记（舞蹈）"""
    LogicNoteVisual = 8
    """ロジックノート（ビジュアル）理性笔记（形象）"""
    AnomalyNoteVocal = 9
    """アノマリーノート（ボーカル）非凡笔记（声乐）"""
    AnomalyNoteDance = 10
    """アノマリーノート（ダンス）非凡笔记（舞蹈）"""
    AnomalyNoteVisual = 11
    """アノマリーノート（ビジュアル）非凡笔记（形象）"""
    RechallengeTicket = 12
    """再挑戦チケット 重新挑战券"""
    RecordKey = 13
    """記録の鍵 解锁交流的物品"""

    # 碎片
    IdolPiece_倉本千奈_WonderScale = 14
    """倉本千奈 WonderScale 碎片"""
    IdolPiece_篠泽广_光景 = 15
    """篠泽广 光景 碎片"""
    IdolPiece_紫云清夏_TameLieOneStep = 16
    """紫云清夏 Tame-Lie-One-Step 碎片"""

    @classmethod
    def to_ui_text(cls, item: "DailyMoneyShopItems") -> str:
        """获取枚举值对应的UI显示文本"""
        MAP = {
            cls.Recommendations: "所有推荐商品",
            cls.LessonNote: "课程笔记",
            cls.VeteranNote: "老手笔记",
            cls.SupportEnhancementPt: "支援强化点数",
            cls.SenseNoteVocal: "感性笔记（声乐）",
            cls.SenseNoteDance: "感性笔记（舞蹈）",
            cls.SenseNoteVisual: "感性笔记（形象）",
            cls.LogicNoteVocal: "理性笔记（声乐）",
            cls.LogicNoteDance: "理性笔记（舞蹈）",
            cls.LogicNoteVisual: "理性笔记（形象）",
            cls.AnomalyNoteVocal: "非凡笔记（声乐）",
            cls.AnomalyNoteDance: "非凡笔记（舞蹈）",
            cls.AnomalyNoteVisual: "非凡笔记（形象）",
            cls.RechallengeTicket: "重新挑战券",
            cls.RecordKey: "记录钥匙",
            cls.IdolPiece_倉本千奈_WonderScale: "倉本千奈 WonderScale 碎片",
            cls.IdolPiece_篠泽广_光景: "篠泽广 光景 碎片",
            cls.IdolPiece_紫云清夏_TameLieOneStep: "紫云清夏 Tame-Lie-One-Step 碎片"
        }
        return MAP.get(item, str(item))
    
    @classmethod
    def all(cls) -> list[tuple[str, 'DailyMoneyShopItems']]:
        """获取所有枚举值及其对应的UI显示文本"""
        return [(cls.to_ui_text(item), item) for item in cls]

    def to_resource(self):
        from . import R
        match self:
            case DailyMoneyShopItems.Recommendations:
                return R.Daily.TextShopRecommended
            case DailyMoneyShopItems.LessonNote:
                return R.Shop.ItemLessonNote
            case DailyMoneyShopItems.VeteranNote:
                return R.Shop.ItemVeteranNote
            case DailyMoneyShopItems.SupportEnhancementPt:
                return R.Shop.ItemSupportEnhancementPt
            case DailyMoneyShopItems.SenseNoteVocal:
                return R.Shop.ItemSenseNoteVocal
            case DailyMoneyShopItems.SenseNoteDance:
                return R.Shop.ItemSenseNoteDance
            case DailyMoneyShopItems.SenseNoteVisual:
                return R.Shop.ItemSenseNoteVisual
            case DailyMoneyShopItems.LogicNoteVocal:
                return R.Shop.ItemLogicNoteVocal
            case DailyMoneyShopItems.LogicNoteDance:
                return R.Shop.ItemLogicNoteDance
            case DailyMoneyShopItems.LogicNoteVisual:
                return R.Shop.ItemLogicNoteVisual
            case DailyMoneyShopItems.AnomalyNoteVocal:
                return R.Shop.ItemAnomalyNoteVocal
            case DailyMoneyShopItems.AnomalyNoteDance:
                return R.Shop.ItemAnomalyNoteDance
            case DailyMoneyShopItems.AnomalyNoteVisual:
                return R.Shop.ItemAnomalyNoteVisual
            case DailyMoneyShopItems.RechallengeTicket:
                return R.Shop.ItemRechallengeTicket
            case DailyMoneyShopItems.RecordKey:
                return R.Shop.ItemRecordKey
            case DailyMoneyShopItems.IdolPiece_倉本千奈_WonderScale:
                return R.Shop.IdolPiece.倉本千奈_WonderScale
            case DailyMoneyShopItems.IdolPiece_篠泽广_光景:
                return R.Shop.IdolPiece.篠泽广_光景
            case DailyMoneyShopItems.IdolPiece_紫云清夏_TameLieOneStep:
                return R.Shop.IdolPiece.紫云清夏_TameLieOneStep
            case _:
                raise ValueError(f"Unknown daily shop item: {self}")

class ConfigBaseModel(BaseModel):
    model_config = ConfigDict(use_attribute_docstrings=True)

class PurchaseConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用商店购买"""
    money_enabled: bool = False
    """是否启用金币购买"""
    money_items: list[DailyMoneyShopItems] = []
    """金币商店要购买的物品"""
    money_refresh_on: Literal['never', 'not_found', 'always'] = 'never'
    """
    金币商店刷新逻辑。

    * never: 从不刷新。
    * not_found: 仅当要购买的物品不存在时刷新。
    * always: 总是刷新。
    """
    ap_enabled: bool = False
    """是否启用AP购买"""
    ap_items: list[Literal[0, 1, 2, 3]] = []
    """AP商店要购买的物品"""


class ActivityFundsConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用收取活动费"""


class PresentsConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用收取礼物"""


class AssignmentConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用工作"""

    mini_live_reassign_enabled: bool = False
    """是否启用重新分配 MiniLive"""
    mini_live_duration: Literal[4, 6, 12] = 12
    """MiniLive 工作时长"""

    online_live_reassign_enabled: bool = False
    """是否启用重新分配 OnlineLive"""
    online_live_duration: Literal[4, 6, 12] = 12
    """OnlineLive 工作时长"""


class ContestConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用竞赛"""

class ProduceConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用培育"""
    mode: Literal['regular', 'pro'] = 'regular'
    """
    培育模式。
    进行一次 REGULAR 培育需要 ~30min，进行一次 PRO 培育需要 ~1h。
    """
    produce_count: int = 1
    """培育的次数。"""
    idols: list[PIdol] = []
    """
    要培育的偶像。将会按顺序循环选择培育。
    若未选择任何偶像，则使用游戏默认选择的偶像（为上次培育偶像）。
    """
    memory_sets: list[int] = []
    """要使用的回忆编成编号，从 1 开始。将会按顺序循环选择使用。"""
    support_card_sets: list[int] = []

    """要使用的支援卡编成编号，从 1 开始。将会按顺序循环选择使用。"""
    auto_set_memory: bool = False
    """是否自动编成回忆。此选项优先级高于回忆编成编号。"""
    auto_set_support_card: bool = False
    """是否自动编成支援卡。此选项优先级高于支援卡编成编号。"""
    use_pt_boost: bool = False
    """是否使用支援强化 Pt 提升。"""
    use_note_boost: bool = False
    """是否使用笔记数提升。"""
    follow_producer: bool = False
    """是否关注租借了支援卡的制作人。"""


class MissionRewardConfig(ConfigBaseModel):
    enabled: bool = False
    """是否启用领取任务奖励"""


class BaseConfig(ConfigBaseModel):
    purchase: PurchaseConfig = PurchaseConfig()
    """商店购买配置"""

    activity_funds: ActivityFundsConfig = ActivityFundsConfig()
    """活动费配置"""

    presents: PresentsConfig = PresentsConfig()
    """收取礼物配置"""

    assignment: AssignmentConfig = AssignmentConfig()
    """工作配置"""

    contest: ContestConfig = ContestConfig()
    """竞赛配置"""

    produce: ProduceConfig = ProduceConfig()
    """培育配置"""

    mission_reward: MissionRewardConfig = MissionRewardConfig()
    """领取任务奖励配置"""



def conf() -> BaseConfig:
    """获取当前配置数据"""
    c = config.to(BaseConfig).current
    return c.options

def sprite_path(path: str) -> str:
    standalone = os.path.join('kotonebot/tasks/sprites', path)
    if os.path.exists(standalone):
        return standalone
    return str(resources.files('kotonebot.tasks.sprites') / path)


if __name__ == '__main__':
    print(PurchaseConfig.model_fields['money_refresh_on'].description)


"""工作。お仕事"""
import logging
from typing import Literal
from datetime import timedelta

from . import R
from .common import conf
from .actions.loading import wait_loading_end
from .actions.scenes import at_home, goto_home
from kotonebot import task, device, image, action, ocr, contains, cropped, rect_expand, color, sleep, regex

logger = logging.getLogger(__name__)

@action('领取工作奖励')
def handle_claim_assignment():
    """
    领取工作奖励

    前置条件：点击了工作按钮，已进入领取页面 \n
    结束状态：分配工作页面
    """
    # 领取奖励 [screenshots/assignment/acquire.png]
    if image.find(R.Common.ButtonCompletion):
        device.click()
        return True
    return False

@action('重新分配工作')
def assign(type: Literal['mini', 'online']) -> bool:
    """
    分配工作

    前置条件：分配工作页面 \n
    结束状态：工作开始动画

    :param type: 工作类型。mini=ミニライブ 或 online=ライブ配信。
    """
    # [kotonebot/tasks/assignment.py]
    target_duration = 12
    image.expect_wait(R.Daily.IconTitleAssign, timeout=10)
    if type == 'mini':
        target_duration = conf().assignment.mini_live_duration
        if image.find(R.Daily.IconAssignMiniLive):
            device.click()
        else:
            logger.warning('MiniLive already assigned. Skipping...')
            return False
    elif type == 'online':
        target_duration = conf().assignment.online_live_duration
        if image.find(R.Daily.IconAssignOnlineLive):
            device.click()
        else:
            logger.warning('OnlineLive already assigned. Skipping...')
            return False
    else:
        raise ValueError(f'Invalid type: {type}')
    # MiniLive/OnlineLive 页面 [screenshots/assignment/assign_mini_live.png]
    image.expect_wait(R.Common.ButtonSelect, timeout=5)
    # 选择好调偶像
    selected = False
    max_attempts = 4
    attempts = 0
    while not selected:
        # 寻找所有好调图标
        results = image.find_all(R.Daily.IconAssignKouchou, threshold=0.8)
        results.sort(key=lambda r: tuple(r.position))
        results.pop(0) # 第一个是说明文字里的图标
        # 尝试点击所有目标
        for target in results:
            with cropped(device, y2=0.3):
                img1 = device.screenshot()
                # 选择偶像并判断是否选择成功
                device.click(target)
                sleep(1)
                img2 = device.screenshot()
                if image.raw().similar(img1, img2, 0.97):
                    logger.info(f'Idol #{target} already assigned. Trying next.')
                    continue
                selected = True
                break
        if not selected:
            attempts += 1
            if attempts >= max_attempts:
                logger.warning('Failed to select kouchou idol. Keep using the default idol.')
                break
            # 说明可能在第二页
            device.swipe_scaled(0.6, 0.7, 0.2, 0.7)
            sleep(0.5)
        else:
            break
    # 点击选择
    sleep(0.5)
    device.click(image.expect(R.Common.ButtonSelect))
    # 等待页面加载
    confirm = image.expect_wait(R.Common.ButtonConfirmNoIcon)
    # 选择时间 [screenshots/assignment/assign_mini_live2.png]
    if ocr.find(contains(f'{target_duration}時間')):
        logger.info(f'{target_duration}時間 selected.')
        device.click()
    else:
        logger.warning(f'{target_duration}時間 not found. Using default duration.')
    sleep(0.5)
    # 点击 决定する
    device.click(confirm)
    # 点击 開始する [screenshots/assignment/assign_mini_live3.png]
    # device.click(image.expect_wait(R.Common.ButtonStart, timeout=5))
    # TODO: SimpleDispatcher
    while True:
        device.screenshot()
        if at_assignment():
            break
        if image.find(R.Common.ButtonStart):
            device.click()
            continue
    return True

@action('获取剩余时间')
def get_remaining_time() -> timedelta | None:
    """
    获取剩余时间

    前置条件：首页 \n
    结束状态：-
    """
    texts = ocr.ocr(rect=R.Daily.BoxHomeAssignment)
    if not texts.where(contains('お仕事')):
        logger.warning('お仕事 area not found.')
        return None
    time = texts.where(regex(r'\d+:\d+:\d+')).first()
    if not time:
        logger.warning('お仕事 remaining time not found.')
        return None
    logger.info(f'お仕事 remaining time: {time}')
    return timedelta(hours=time.numbers()[0], minutes=time.numbers()[1], seconds=time.numbers()[2])

@action('检测工作页面')
def at_assignment():
    """
    判断是否在工作页面
    """
    # 不能以 R.Daily.IconTitleAssign 作为判断依据，
    # 因为标题出现后还有一段动画
    return image.find_multi([
        R.Daily.ButtonAssignmentShortenTime,
        R.Daily.IconAssignMiniLive,
        R.Daily.IconAssignOnlineLive,
    ]) is not None

@task('工作')
def assignment():
    """领取工作奖励并重新分配工作"""
    if not conf().assignment.enabled:
        logger.info('Assignment is disabled.')
        return
    if not at_home():
        goto_home()
    btn_assignment = image.expect_wait(R.Daily.ButtonAssignmentPartial)
    notification_rect = rect_expand(btn_assignment.rect, top=40, right=40)
    complete_rect = rect_expand(btn_assignment.rect, right=40, bottom=60)
    with device.pinned():
        completed = color.find_rgb('#ff6085', rect=complete_rect)
        if completed:
            logger.info('Assignment completed. Acquiring...')
        notification_dot = color.find_rgb('#ff134a', rect=notification_rect)
        if not notification_dot and not completed:
            logger.info('No action needed.')
            # TODO: 获取剩余时间，并根据时间更新调度
            return

    # 点击工作按钮
    logger.debug('Clicking assignment icon.')
    device.click(btn_assignment)
    # 等待加载、领取奖励
    while not at_assignment():
        if completed and handle_claim_assignment():
            logger.info('Assignment acquired.')
    # 重新分配
    if conf().assignment.mini_live_reassign_enabled:
        if image.find(R.Daily.IconAssignMiniLive):
            assign('mini')
    else:
        logger.info('MiniLive reassign is disabled.')
    while not at_assignment():
        pass
    if conf().assignment.online_live_reassign_enabled:
        if image.find(R.Daily.IconAssignOnlineLive):
            assign('online')
    else:
        logger.info('OnlineLive reassign is disabled.')
    # 等待动画结束
    while not at_assignment():
        pass

if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.INFO, format='[%(asctime)s] [%(levelname)s] [%(name)s] [%(funcName)s] [%(lineno)d] %(message)s')
    logger.setLevel(logging.DEBUG)
    # assignment()
    print(get_remaining_time())



from dataclasses import dataclass
from typing import Literal, NamedTuple

import numpy as np

from . import R
from kotonebot import action, device, color, image, ocr, sleep
from kotonebot.backend.color import HsvColor
from kotonebot.backend.util import Rect
from kotonebot.backend.core import HintBox, Image

import cv2
from cv2.typing import MatLike

@action('按钮是否禁用', screenshot_mode='manual-inherit')
def button_state(*, target: Image | None = None, rect: Rect | None = None) -> bool | None:
    """
    判断按钮是否处于禁用状态。

    :param rect: 按钮的矩形区域。必须包括文字或图标部分。
    :param target: 按钮目标模板。
    """
    img = device.screenshot()
    if rect is not None:
        _rect = rect
    elif target is not None:
        result = image.find(target)
        if result is None:
            return None
        _rect = result.rect
    else:
        raise ValueError('Either rect or target must be provided.')
    if color.find_rgb('#babcbd', rect=_rect):
        return False
    elif color.find_rgb('#ffffff', rect=_rect):
        return True
    else:
        raise ValueError(f'Unknown button state: {img}')

def web2cv(hsv: HsvColor):
    return (int(hsv[0]/360*180), int(hsv[1]/100*255), int(hsv[2]/100*255))

WHITE_LOW = (0, 0, 200)
WHITE_HIGH = (180, 30, 255)

PINK_TARGET = (335, 78, 95)
PINK_LOW = (300, 70, 90)
PINK_HIGH = (350, 80, 100)

BLUE_TARGET = (210, 88, 93)
BLUE_LOW = (200, 80, 90)
BLUE_HIGH = (220, 90, 100)

YELLOW_TARGET = (39, 81, 97)
YELLOW_LOW = (30, 70, 90)
YELLOW_HIGH = (45, 90, 100)

DEFAULT_COLORS = [
    (web2cv(PINK_LOW), web2cv(PINK_HIGH)),
    (web2cv(YELLOW_LOW), web2cv(YELLOW_HIGH)),
    (web2cv(BLUE_LOW), web2cv(BLUE_HIGH)),
]

def filter_rectangles(
    img: MatLike,
    color_ranges: tuple[HsvColor, HsvColor],
    aspect_ratio_threshold: float,
    area_threshold: int,
    rect: Rect | None = None
) -> list[Rect]:
    """
    过滤出指定颜色，并执行轮廓查找，返回符合要求的轮廓的 bound box。
    返回结果按照 y 坐标排序。
    """
    img_hsv =cv2.cvtColor(img, cv2.COLOR_BGR2HSV)

    white_mask = cv2.inRange(img_hsv, np.array(color_ranges[0]), np.array(color_ranges[1]))
    contours, _ = cv2.findContours(white_mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    result_rects = []
    for contour in contours:
        x, y, w, h = cv2.boundingRect(contour)
        # 如果不在指定范围内，跳过
        if rect is not None:
            rect_x1, rect_y1, rect_w, rect_h = rect
            rect_x2 = rect_x1 + rect_w
            rect_y2 = rect_y1 + rect_h
            if not (
                x >= rect_x1 and
                y >= rect_y1 and
                x + w <= rect_x2 and
                y + h <= rect_y2
            ):
                continue
        aspect_ratio = w / h
        area = cv2.contourArea(contour)
        if aspect_ratio >= aspect_ratio_threshold and area >= area_threshold:
            result_rects.append((x, y, w, h))
    result_rects.sort(key=lambda x: x[1])
    return result_rects

@dataclass
class EventButton:
    rect: Rect
    selected: bool
    description: str
    title: str

# 参考图片：
# [screenshots/produce/action_study3.png]
class CommuEventButtonUI:
    def __init__(
        self,
        selected_colors: list[tuple[HsvColor, HsvColor]] = DEFAULT_COLORS,
        rect: HintBox = R.InPurodyuusu.BoxCommuEventButtonsArea
    ):
        """
        :param selected_colors: 按钮选中后的主题色。
        :param rect: 识别范围
        """
        self.color_ranges = selected_colors
        self.rect = rect

    @action('交流事件按钮.识别选中', screenshot_mode='manual-inherit')
    def selected(self, description: bool = True, title: bool = False) -> EventButton | None:
        img = device.screenshot()
        for i, color_range in enumerate(self.color_ranges):
            rects = filter_rectangles(img, color_range, 7, 500, rect=self.rect)
            if len(rects) > 0:
                desc_text = self.description() if description else ''
                title_text = ocr.ocr(rect=rects[0]).squash().text if title else ''
                return EventButton(rects[0], True, desc_text, title_text)
        return None

    @action('交流事件按钮.识别按钮', screenshot_mode='manual-inherit')
    def all(self, description: bool = True, title: bool = False) -> list[EventButton]:
        """
        识别所有按钮的位置以及选中后的描述文本

        前置条件：当前显示了交流事件按钮\n
        结束状态：-

        :param description: 是否识别描述文本。
        :param title: 是否识别标题。
        """
        img = device.screenshot()
        rects = filter_rectangles(img, (WHITE_LOW, WHITE_HIGH), 7, 500, rect=self.rect)
        if not rects:
            return []
        selected = self.selected()
        result: list[EventButton] = []
        for rect in rects:
            desc_text = ''
            title_text = ''
            if title:
                title_text = ocr.ocr(rect=rect).squash().text
            if description:
                device.click(rect)
                sleep(0.15)
                device.screenshot()
                desc_text = self.description()
            result.append(EventButton(rect, False, desc_text, title_text))
        # 修改最后一次点击的按钮为 selected 状态
        if len(result) > 0:
            result[-1].selected = True
        if selected is not None:
            result.append(selected)
            selected.selected = False
        result.sort(key=lambda x: x.rect[1])
        return result

    @action('交流事件按钮.识别描述', screenshot_mode='manual-inherit')
    def description(self) -> str:
        """
        识别当前选中按钮的描述文本

        前置条件：有选中按钮\n
        结束状态：-
        """
        img = device.screenshot()
        rects = filter_rectangles(img, (WHITE_LOW, WHITE_HIGH), 3, 1000, rect=self.rect)
        rects.sort(key=lambda x: x[1])
        ocr_result = ocr.raw().ocr(img, rect=rects[0])
        return ocr_result.squash().text

def filter_white(img: MatLike):
    hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    lower_white = np.array([0, 0, 180])
    upper_white = np.array([180, 50, 255])
    return cv2.inRange(hsv, lower_white, upper_white)

# TODO: image 对象加入自定义 hook，处理 post-process 和 pre-process
@action('工具栏按钮.寻找首页', screenshot_mode='manual-inherit')
def toolbar_home():
    img = device.screenshot()
    img = filter_white(img)
    result = image.raw().find(img, R.Common.ButtonToolbarHomeBinary.binary())
    return result


if __name__ == '__main__':
    from pprint import pprint as print
    from kotonebot.backend.context import init_context, manual_context, device
    init_context()
    manual_context().begin()
    print(toolbar_home())


import os
import re
import json
import time
import uuid
import psutil
import hashlib
import traceback
from pathlib import Path
from functools import cache
from datetime import datetime
from dataclasses import dataclass
from typing import NamedTuple, TextIO, Literal

import cv2
from cv2.typing import MatLike

from ..core import Image

class Result(NamedTuple):
    title: str
    image: list[str]
    description: str

@dataclass
class _Vars:
    """调试变量类"""
    enabled: bool = False
    """是否启用调试结果显示。"""

    max_results: int = -1
    """最多保存的结果数量。-1 表示不限制。"""

    wait_for_message_sent: bool = False
    """
    是否等待消息发送完成才继续后续代码。

    默认禁用。启用此选项会显著降低运行速度。
    """
    
    hide_server_log: bool = True
    """是否隐藏服务器日志。"""

    auto_save_to_folder: str | None = None
    """
    是否将结果自动保存到指定文件夹。
    
    如果为 None，则不保存。
    """

    hash_image: bool = True
    """
    是否使用图片的 MD5 值作为图片的唯一标识。
    若禁用，则使用随机 UUID 作为图片的唯一标识
    （可能会导致保存大量重复图片）。
    
    此选项默认启用。启用此选项会轻微降低调试时运行速度。
    """

debug = _Vars()

# TODO: 需要考虑释放内存的问题。释放哪些比较合适？
_results: dict[str, Result] = {}
_images: dict[str, MatLike] = {}
"""存放临时图片的字典。"""
_result_file: TextIO | None = None

def _save_image(image: MatLike | Image) -> str:
    """缓存图片数据到 _images 字典中。返回 key。"""
    if isinstance(image, Image):
        image = image.data
    # 计算 key
    if debug.hash_image:
        key = hashlib.md5(image.tobytes()).hexdigest()
    else:
        key = str(uuid.uuid4())
    # 保存图片
    if key not in _images:
        _images[key] = image
        if debug.auto_save_to_folder:
            if not os.path.exists(debug.auto_save_to_folder):
                os.makedirs(debug.auto_save_to_folder)
            file_name = f"{key}.png"
            cv2.imwrite(os.path.join(debug.auto_save_to_folder, file_name), image)
    return key

def _save_images(images: list[MatLike]) -> list[str]:
    """缓存图片数据到 _images 字典中。返回 key 列表。"""
    return [_save_image(image) for image in images]

def img(image: str | MatLike | Image | None) -> str:
    """
    用于在 `result()` 函数中嵌入图片。

    :param image: 图片路径或 OpenCV 图片对象。
    :return: 图片的 HTML 代码。
    """
    if image is None:
        return 'None'
    if debug.auto_save_to_folder:
        if isinstance(image, str):
            image = cv2.imread(image)
        elif isinstance(image, Image):
            image = image.data
        key = _save_image(image)
        return f'[img]{key}[/img]'
    else:
        if isinstance(image, str):
            return f'<img src="/api/read_file?path={image}" />'
        elif isinstance(image, Image) and image.path:
            return f'<img src="/api/read_file?path={image.path}" />'
        else:
            key = _save_image(image)
            return f'<img src="/api/read_memory?key={key}" />'

def color(color: str | tuple[int, int, int] | None) -> str:
    """
    用于在调试结果中嵌入颜色。
    """
    if color is None:
        return 'None'
    if isinstance(color, tuple):
        color = '#{:02X}{:02X}{:02X}'.format(color[0], color[1], color[2])
        return f'<kbd-color style="display:inline-block; white-space:initial;" color="{color}"></kbd-color>'
    else:
        return f'<kbd-color style="display:inline-block; white-space:initial;" color="{color}"></kbd-color>'

def to_html(text: str) -> str:
    """将文本转换为 HTML 代码。"""
    text = text.replace('<', '&lt;').replace('>', '&gt;')
    text = text.replace('\n', '<br>')
    text = text.replace(' ', '&nbsp;')
    return text

IDEType = Literal['vscode', 'cursor', 'windsurf']

@cache
def get_current_ide() -> IDEType | None:
    """获取当前IDE类型"""
    me = psutil.Process()
    while True:
        parent = me.parent()
        if parent is None:
            break
        name = parent.name()
        if name.lower() == 'code.exe':
            return 'vscode'
        elif name.lower() == 'cursor.exe':
            return 'cursor'
        elif name.lower() == 'windsurf.exe':
            return 'windsurf'
        me = parent
    return None

def _make_code_file_url(
    text: str,
    full_path: str,
    line: int = 0,
) -> str:
    """
    将代码文本转换为 VSCode 的文件 URL。
    """
    ide = get_current_ide()
    if ide == 'vscode':
        prefix = 'vscode'
    elif ide == 'cursor':
        prefix = 'cursor'
    elif ide == 'windsurf':
        prefix = 'windsurf'
    else:
        return text
    url = f"{prefix}://file/{full_path}:{line}:0"
    return f'<a href="{url}">{text}</a>'

def result(
        title: str,
        image: MatLike | list[MatLike],
        text: str = ''
    ):
    """
    显示图片结果。

    例：
    ```python
    result(
        "image.find",
        image,
        f"template: {img(template)} \\n"
        f"matches: {len(matches)} \\n"
    )
    ```
    
    :param title: 标题。建议使用 `模块.方法` 格式。
    :param image: 图片。
    :param text: 详细文本。可以是 HTML 代码，空格和换行将会保留。如果需要嵌入图片，使用 `img()` 函数。
    """
    global _result_file
    if not debug.enabled:
        return
    if not isinstance(image, list):
        image = [image]
    
    key = 'result_' + title + '_' + str(time.time())
    # 保存图片
    saved_images = _save_images(image)
    _results[key] = Result(title, saved_images, text)
    if len(_results) > debug.max_results:
        _results.pop(next(iter(_results)))
    # 拼接消息
    now_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-4]
    # 获取完整堆栈
    callstack = []
    for frame in traceback.format_stack():
        if not re.search(r'Python\d*[\/\\]lib|debugpy', frame):
            # 提取文件路径和行号
            match = re.search(r'File "([^"]+)", line (\d+)', frame)
            if match:
                file_path = match.group(1)
                file_path = to_html(file_path)
                line_num = match.group(2)
                # 将绝对路径转换为相对路径
                rel_path = file_path.replace(str(Path.cwd()), '.')
                # 将文件路径和行号转换为链接
                frame = frame.replace(
                    f'File "{file_path}", line {line_num}',
                    f'File "{_make_code_file_url(rel_path, file_path, int(line_num))}", line {line_num}'
                )
            callstack.append(frame)
    callstack_str = '\n'.join(callstack)

    # 获取简化堆栈(只包含函数名)
    simple_callstack = []
    for frame in traceback.extract_stack():
        if not re.search(r'Python\d*[\/\\]lib|debugpy', frame.filename):
            module = Path(frame.filename).stem # 只获取文件名,不含路径和扩展名
            simple_callstack.append(f"{module}.{frame.name}")
    simple_callstack_str = ' -> '.join(simple_callstack)
    simple_callstack_str = to_html(simple_callstack_str)

    final_text = (
        f"Time: {now_time}<br>" +
        f"Callstack: {simple_callstack_str}<br>" +
        f"<details><summary>Full Callstack</summary>{callstack_str}</details><br>" +
        f"<hr>{text}"
    )
    # 发送 WS 消息
    from .server import send_ws_message
    send_ws_message(title, saved_images, final_text, wait=debug.wait_for_message_sent)

    # 保存到文件
    # TODO: 把这个类型转换为 dataclass/namedtuple
    if debug.auto_save_to_folder:
        if _result_file is None:
            if not os.path.exists(debug.auto_save_to_folder):
                os.makedirs(debug.auto_save_to_folder)
            log_file_name = f"dump_{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}.json"
            _result_file = open(os.path.join(debug.auto_save_to_folder, log_file_name), "w")
        _result_file.write(json.dumps({
            "image": {
                "type": "memory",
                "value": saved_images
            },
            "name": title,
            "details": final_text
        }))
        _result_file.write("\n")




"""启动游戏，领取登录奖励，直到首页为止"""
import logging

from kotonebot import task, device, image, cropped, AdaptiveWait, sleep
from . import R
from .common import Priority
from .actions.loading import loading
from .actions.scenes import at_home, goto_home
from .actions.commu import is_at_commu, handle_unread_commu
logger = logging.getLogger(__name__)

@task('启动游戏', priority=Priority.START_GAME)
def start_game():
    """
    启动游戏，直到游戏进入首页为止。
    
    执行前游戏必须处于未启动状态。
    """
    # TODO: 包名放到配置文件里
    if device.current_package() == 'com.bandainamcoent.idolmaster_gakuen':
        logger.info("Game already started")
        if not at_home():
            logger.info("Not at home, going to home")
            goto_home()
        return
    device.launch_app('com.bandainamcoent.idolmaster_gakuen')
    # [screenshots/startup/1.png]
    image.wait_for(R.Daily.ButonLinkData, timeout=30)
    sleep(2)
    device.click_center()
    wait = AdaptiveWait(timeout=240, timeout_message='Game startup loading timeout')
    while True:
        while loading():
            wait()
        with device.pinned():
            if image.find(R.Daily.ButtonHomeCurrent):
                break
            # [screenshots/startup/update.png]
            elif image.find(R.Common.TextGameUpdate):
                device.click(image.expect(R.Common.ButtonConfirm))
            # [screenshots/startup/announcement1.png]
            elif image.find(R.Common.ButtonIconClose):
                device.click()
            # [screenshots/startup/birthday.png]
            elif handle_unread_commu():
                pass
            else:
                device.click_center()
            wait()

if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.INFO, format='[%(asctime)s] [%(levelname)s] [%(name)s] [%(funcName)s] [%(lineno)d] %(message)s')
    logger.setLevel(logging.DEBUG)
    start_game()




import logging
from functools import cache
from typing import Callable, overload, TYPE_CHECKING

import cv2
from cv2.typing import MatLike

from kotonebot.errors import ResourceFileMissingError
if TYPE_CHECKING:
    from kotonebot.backend.util import Rect

class Ocr:
    def __init__(
        self,
        text: str | Callable[[str], bool],
        *,
        language: str = 'jp',
    ):
        self.text = text
        self.language = language


class Image:
    def __init__(
        self,
        *,
        path: str | None = None,
        name: str | None = 'untitled',
        data: MatLike | None = None,
    ):
        self.path = path
        self.name = name
        self.__data: MatLike | None = data
        self.__data_with_alpha: MatLike | None = None

    @cache
    def binary(self) -> 'Image':
        return Image(data=cv2.cvtColor(self.data, cv2.COLOR_BGR2GRAY))

    @property
    def data(self) -> MatLike:
        if self.__data is None:
            if self.path is None:
                raise ValueError('Either path or data must be provided.')
            self.__data = cv2.imread(self.path)
            if self.__data is None:
                raise ResourceFileMissingError(self.path, 'sprite')
            logger.debug(f'Read image "{self.name}" from {self.path}')
        return self.__data
    
    @property
    def data_with_alpha(self) -> MatLike:
        if self.__data_with_alpha is None:
            if self.path is None:
                raise ValueError('Either path or data must be provided.')
            self.__data_with_alpha = cv2.imread(self.path, cv2.IMREAD_UNCHANGED)
            if self.__data_with_alpha is None:
                raise ResourceFileMissingError(self.path, 'sprite with alpha')
            logger.debug(f'Read image "{self.name}" from {self.path}')
        return self.__data_with_alpha
    
    def __repr__(self) -> str:
        if self.path is None:
            return f'<Image: memory>'
        else:
            return f'<Image: "{self.name}" at {self.path}>'


class HintBox(tuple[int, int, int, int]):
    def __new__(
        cls,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
        *,
        source_resolution: tuple[int, int],
    ):
        w = x2 - x1
        h = y2 - y1
        return super().__new__(cls, [x1, y1, w, h])
    
    def __init__(
        self,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
        *,
        name: str | None = None,
        description: str | None = None,
        source_resolution: tuple[int, int],
    ):
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.name = name
        self.description = description
        self.source_resolution = source_resolution

    @property
    def width(self) -> int:
        return self.x2 - self.x1
    
    @property
    def height(self) -> int:
        return self.y2 - self.y1
    
    @property
    def rect(self) -> 'Rect':
        return self.x1, self.y1, self.width, self.height


logger = logging.getLogger(__name__)


@overload
def image(data: str) -> Image:


    """从文件路径创建 Image 对象。"""
    ...
@overload
def image(data: MatLike) -> Image:
    """从 OpenCV 的 MatLike 对象创建 Image 对象。"""
    ...

def image(data: str | MatLike) -> Image:
    if isinstance(data, str):
        return Image(path=data)
    else:
        return Image(data=data)
 
def ocr(text: str | Callable[[str], bool], language: str = 'jp') -> Ocr:
    return Ocr(text, language=language)

if __name__ == '__main__':
    hint_box = HintBox(100, 100, 200, 200, source_resolution=(1920, 1080))
    print(hint_box.rect)
    print(hint_box.width)
    print(hint_box.height)




"""领取任务奖励"""
import logging

from . import R
from .common import conf, Priority
from .actions.loading import wait_loading_end
from .actions.scenes import at_home, goto_home
from kotonebot import device, image, color, task, action, rect_expand, sleep

logger = logging.getLogger(__name__)

@action('检查任务')
def check_and_goto_mission() -> bool:
    """
    检查任务。如果需要领取，
    则前往任务页面，并返回 True。
    否则返回 False。
    
    :return: 是否需要领取任务奖励
    """
    rect = image.expect_wait(R.Daily.ButtonMission, timeout=1).rect
    # 向上、向右扩展 50px
    color_rect = rect_expand(rect, top=50, right=50)
    if not color.find_rgb('#ff1249', rect=color_rect):
        logger.info('No mission reward to claim.')
        return False
    # 点击任务奖励图标
    logger.debug('Clicking mission reward icon.')
    device.click()
    sleep(0.5)
    # 加载
    wait_loading_end()
    return True

@action('任务奖励')
def claim_mission_reward(name: str):
    """领取任务奖励"""
    # [screenshots/mission/daily.png]
    image.expect_wait(R.Common.ButtonIconArrowShort)
    if image.find(R.Common.ButtonIconArrowShort, colored=True):
        logger.info(f'Claiming {name} mission reward.')
        device.click()
        sleep(0.5)
        while not image.find(R.Common.ButtonIconArrowShortDisabled, colored=True):
            if image.find(R.Common.ButtonIconClose):
                logger.debug('Closing popup dialog.')
                device.click()
                sleep(1)
            sleep(0.5)
    else:
        logger.info(f'No {name} mission reward to claim.')

@action('领取任务页面奖励')
def claim_mission_rewards():
    """领取任务奖励"""
    # [screenshots/mission/daily.png]
    # デイリー Daily
    claim_mission_reward('デイリー')
    # ウィークリー Weekly
    device.swipe_scaled(0.8, 0.5, 0.2, 0.5)
    sleep(0.5)
    claim_mission_reward('ウィークリー')
    # ノーマル Normal
    device.swipe_scaled(0.8, 0.5, 0.2, 0.5)
    sleep(0.5)
    claim_mission_reward('ノーマル')
    # 期間限定
    device.swipe_scaled(0.8, 0.5, 0.2, 0.5)
    sleep(0.5)
    claim_mission_reward('期間限定')
    

@action('通行证奖励')
def claim_pass_reward():
    """领取通行证奖励"""
    # [screenshots/mission/daily.png]
    pass_rect = image.expect_wait(R.Daily.ButtonIconPass, timeout=1).rect
    # 向右扩展 150px，向上扩展 35px
    color_rect = (pass_rect[0], pass_rect[1] - 35, pass_rect[2] + 150, pass_rect[3] + 35)
    if not color.find_rgb('#ff1249', rect=color_rect):
        logger.info('No pass reward to claim.')
        return
    logger.info('Claiming pass reward.')
    logger.debug('Clicking パス button.')
    device.click()
    sleep(0.5)
    # [screenshots/mission/pass.png]
    # 对话框 [screenshots/mission/pass_dialog.png]
    while not image.find(R.Daily.IconTitlePass):
        if image.find(R.Common.ButtonIconClose):
            logger.debug('Closing popup dialog.')
            device.click()
            sleep(1)
        sleep(0.5)
    logger.debug('Pass screen loaded.')
    while image.find(R.Daily.ButtonPassClaim, colored=True):
        logger.debug('Clicking 受取 button.')
        device.click()
        sleep(1.5)
        while not image.find(R.Daily.ButtonPassClaim):
            if image.find(R.Common.ButtonIconClose):
                logger.debug('Closing popup dialog.')
                device.click()
                sleep(1)
            sleep(0.5)
        sleep(1.5)
    logger.info('All pass rewards claimed.')


@action('活动奖励')
def claim_event_reward():
    """领取活动奖励"""
    # TODO: 领取活动奖励
    pass

@task('领取任务奖励', priority=Priority.CLAIM_MISSION_REWARD)
def mission_reward():
    """
    领取任务奖励
    """
    if not conf().mission_reward.enabled:
        logger.info('Mission reward is disabled.')
        return
    logger.info('Claiming mission rewards.')
    if not at_home():
        goto_home()
    if not check_and_goto_mission():
        return
    sleep(0.7)
    claim_mission_rewards()
    sleep(0.5)
    claim_pass_reward()
    logger.info('All mission rewards claimed.')


if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.INFO, format='[%(asctime)s] [%(levelname)s] [%(name)s] [%(funcName)s] [%(lineno)d] %(message)s')
    logging.getLogger('kotonebot').setLevel(logging.DEBUG)
    logger.setLevel(logging.DEBUG)
    
    # if image.find(R.Common.CheckboxUnchecked):
    #     logger.debug('Checking skip all.')
    #     device.click()
    #     sleep(0.5)
    # device.click(image.expect(R.Daily.ButtonIconSkip, colored=True, transparent=True, threshold=0.999))
    # mission_reward()
    claim_pass_reward()



from typing import Protocol, TYPE_CHECKING, runtime_checkable, Literal

from cv2.typing import MatLike

from kotonebot.backend.util import Rect
if TYPE_CHECKING:
    from .device import Device

@runtime_checkable
class ClickableObjectProtocol(Protocol):
    """
    可点击对象的协议
    """
    @property
    def rect(self) -> Rect:
        ...

class DeviceScreenshotProtocol(Protocol):
    def screenshot(self) -> MatLike:
        """
        截图
        """
        ...

@runtime_checkable
class Commandable(Protocol):
    def __init__(self, device: 'Device'): ...
    def launch_app(self, package_name: str) -> None: ...
    def current_package(self) -> str | None: ...

@runtime_checkable
class Screenshotable(Protocol):
    def __init__(self, device: 'Device'): ...
    @property
    def screen_size(self) -> tuple[int, int]:
        """
        屏幕尺寸。格式为 `(width, height)`。
        
        **注意**： 此属性返回的分辨率会随设备方向变化。
        如果 `self.orientation` 为 `landscape`，则返回的分辨率是横屏下的分辨率，
        否则返回竖屏下的分辨率。

        `self.orientation` 属性默认为竖屏。如果需要自动检测，
        调用 `self.detect_orientation()` 方法。
        如果已知方向，也可以直接设置 `self.orientation` 属性。
        """
        ...
    
    def detect_orientation(self) -> Literal['portrait', 'landscape'] | None: ...
    def screenshot(self) -> MatLike: ...

@runtime_checkable
class Touchable(Protocol):
    def __init__(self, device: 'Device'): ...
    def click(self, x: int, y: int) -> None: ...
    def swipe(self, x1: int, y1: int, x2: int, y2: int, duration: float|None = None) -> None: ...



import wx
import cv2
import numpy as np
import time
from typing import Optional, Tuple, Callable
from threading import Thread, Lock
from cv2.typing import MatLike
from queue import Queue

from kotonebot.client.device import Device

class DeviceMirrorPanel(wx.Panel):
    def __init__(self, parent, device: Device, log_callback=None):
        super().__init__(parent)
        self.device = device
        self.screen_bitmap: Optional[wx.Bitmap] = None
        self.fps = 0
        self.last_frame_time = time.time()
        self.frame_count = 0
        self.is_running = True
        self.lock = Lock()
        self.last_mouse_pos = (0, 0)
        self.is_dragging = False
        self.screenshot_interval = 0  # 截图耗时(ms)
        self.log_callback = log_callback
        self.operation_queue = Queue()
        
        # 设置背景色为黑色
        self.SetBackgroundColour(wx.BLACK)
        
        # 双缓冲，减少闪烁
        self.SetDoubleBuffered(True)
        
        # 绑定事件
        self.Bind(wx.EVT_PAINT, self.on_paint)
        self.Bind(wx.EVT_SIZE, self.on_size)
        self.Bind(wx.EVT_LEFT_DOWN, self.on_left_down)
        self.Bind(wx.EVT_LEFT_UP, self.on_left_up)
        self.Bind(wx.EVT_MOTION, self.on_motion)
        
        # 启动刷新线程
        self.update_thread = Thread(target=self.update_screen, daemon=True)
        self.update_thread.start()
        
        # 启动操作处理线程
        self.operation_thread = Thread(target=self.process_operations, daemon=True)
        self.operation_thread.start()
        
    def process_operations(self):
        """处理设备操作的线程"""
        while self.is_running:
            try:
                operation = self.operation_queue.get()
                if operation is not None:
                    operation()
                self.operation_queue.task_done()
            except Exception as e:
                if self.log_callback:
                    self.log_callback(f"操作执行错误: {e}")
                    
    def execute_device_operation(self, operation: Callable):
        """将设备操作添加到队列"""
        self.operation_queue.put(operation)

    def update_screen(self):
        while self.is_running:
            try:
                # 获取设备截图并计时
                start_time = time.time()
                frame = self.device.screenshot()
                end_time = time.time()
                self.screenshot_interval = int((end_time - start_time) * 1000)
                
                if frame is None:
                    continue
                    
                # 计算FPS
                current_time = time.time()
                self.frame_count += 1
                if current_time - self.last_frame_time >= 1.0:
                    self.fps = self.frame_count
                    self.frame_count = 0
                    self.last_frame_time = current_time
                
                # 转换为wx.Bitmap
                height, width = frame.shape[:2]
                frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                wximage = wx.Bitmap.FromBuffer(width, height, frame)
                
                with self.lock:
                    self.screen_bitmap = wximage
                
                # 请求重绘
                wx.CallAfter(self.Refresh)
                
                # 控制刷新率
                time.sleep(1/60)
                
            except Exception as e:
                print(f"Error updating screen: {e}")
                time.sleep(1)
    
    def on_paint(self, event):
        dc = wx.BufferedPaintDC(self)
        
        # 清空背景
        dc.SetBackground(wx.Brush(wx.BLACK))
        dc.Clear()
        
        if not self.screen_bitmap:
            return
            
        # 绘制设备画面
        with self.lock:
            # 计算缩放比例，保持宽高比
            panel_width, panel_height = self.GetSize()
            bitmap_width = self.screen_bitmap.GetWidth()
            bitmap_height = self.screen_bitmap.GetHeight()
            
            scale = min(panel_width/bitmap_width, panel_height/bitmap_height)
            scaled_width = int(bitmap_width * scale)
            scaled_height = int(bitmap_height * scale)
            
            # 居中显示
            x = (panel_width - scaled_width) // 2
            y = (panel_height - scaled_height) // 2
            
            if scale != 1:
                img = self.screen_bitmap.ConvertToImage()
                img = img.Scale(scaled_width, scaled_height, wx.IMAGE_QUALITY_HIGH)
                bitmap = wx.Bitmap(img)
            else:
                bitmap = self.screen_bitmap
                
            dc.DrawBitmap(bitmap, x, y)
        
        # 绘制FPS和截图时间
        dc.SetTextForeground(wx.GREEN)
        dc.SetFont(wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        dc.DrawText(f"FPS: {self.fps}", 10, 10)
        dc.DrawText(f"Interval: {self.screenshot_interval}ms", 10, 30)
    
    def on_size(self, event):
        self.Refresh()
        event.Skip()
        
    def get_device_coordinates(self, x: int, y: int) -> Tuple[int, int]:
        """将面板坐标转换为设备坐标"""
        if not self.screen_bitmap:
            return (0, 0)
            
        panel_width, panel_height = self.GetSize()
        bitmap_width = self.screen_bitmap.GetWidth()
        bitmap_height = self.screen_bitmap.GetHeight()
        
        scale = min(panel_width/bitmap_width, panel_height/bitmap_height)
        scaled_width = int(bitmap_width * scale)
        scaled_height = int(bitmap_height * scale)
        
        # 计算显示区域的偏移
        x_offset = (panel_width - scaled_width) // 2
        y_offset = (panel_height - scaled_height) // 2
        
        # 转换坐标
        device_x = int((x - x_offset) / scale)
        device_y = int((y - y_offset) / scale)
        
        # 确保坐标在设备范围内
        device_x = max(0, min(device_x, bitmap_width-1))
        device_y = max(0, min(device_y, bitmap_height-1))
        
        return (device_x, device_y)
    
    def on_left_down(self, event):
        self.last_mouse_pos = event.GetPosition()
        self.is_dragging = True
        event.Skip()
        
    def on_left_up(self, event):
        if not self.is_dragging:
            return
            
        self.is_dragging = False
        pos = event.GetPosition()
        
        # 如果鼠标位置没有明显变化，执行点击
        if abs(pos[0] - self.last_mouse_pos[0]) < 5 and abs(pos[1] - self.last_mouse_pos[1]) < 5:
            device_x, device_y = self.get_device_coordinates(*pos)
            self.execute_device_operation(lambda: self.device.click(device_x, device_y))
            if self.log_callback:
                self.log_callback(f"点击: ({device_x}, {device_y})")
        else:
            # 执行滑动
            start_x, start_y = self.get_device_coordinates(*self.last_mouse_pos)
            end_x, end_y = self.get_device_coordinates(*pos)
            self.execute_device_operation(lambda: self.device.swipe(start_x, start_y, end_x, end_y))
            if self.log_callback:
                self.log_callback(f"滑动: ({start_x}, {start_y}) -> ({end_x}, {end_y})")
            
        event.Skip()
        
    def on_motion(self, event):
        if not self.is_dragging:
            event.Skip()
            return
            
        event.Skip()

class DeviceMirrorFrame(wx.Frame):
    def __init__(self, device: Device):
        super().__init__(None, title="设备镜像", size=(800, 600))
        
        # 创建分割窗口
        self.splitter = wx.SplitterWindow(self)
        
        # 创建左侧面板（包含控制区域和日志区域）
        self.left_panel = wx.Panel(self.splitter)
        left_sizer = wx.BoxSizer(wx.VERTICAL)
        
        # 控制区域
        self.control_panel = wx.Panel(self.left_panel)
        self.init_control_panel()
        left_sizer.Add(self.control_panel, 0, wx.EXPAND | wx.ALL, 5)
        
        # 日志区域
        self.log_text = wx.TextCtrl(self.left_panel, style=wx.TE_MULTILINE | wx.TE_READONLY | wx.HSCROLL)
        self.log_text.SetBackgroundColour(wx.BLACK)
        self.log_text.SetForegroundColour(wx.GREEN)
        self.log_text.SetFont(wx.Font(9, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        left_sizer.Add(self.log_text, 1, wx.EXPAND | wx.ALL, 5)
        
        self.left_panel.SetSizer(left_sizer)
        
        # 创建设备画面
        self.device_panel = DeviceMirrorPanel(self.splitter, device, self.log)
        
        # 设置分割
        self.splitter.SplitVertically(self.left_panel, self.device_panel)
        self.splitter.SetMinimumPaneSize(200)
        
        # 保存设备引用
        self.device = device
        
    def log(self, message: str):
        """添加日志"""
        timestamp = time.strftime("%H:%M:%S", time.localtime())
        wx.CallAfter(self.log_text.AppendText, f"[{timestamp}] {message}\n")
        
    def init_control_panel(self):
        vbox = wx.BoxSizer(wx.VERTICAL)
        
        # 添加控制按钮
        btn_get_resolution = wx.Button(self.control_panel, label="获取分辨率")
        btn_get_resolution.Bind(wx.EVT_BUTTON, self.on_get_resolution)
        vbox.Add(btn_get_resolution, 0, wx.EXPAND | wx.ALL, 5)
        
        btn_get_orientation = wx.Button(self.control_panel, label="获取设备方向")
        btn_get_orientation.Bind(wx.EVT_BUTTON, self.on_get_orientation)
        vbox.Add(btn_get_orientation, 0, wx.EXPAND | wx.ALL, 5)
        
        # 启动APP区域
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.package_input = wx.TextCtrl(self.control_panel)
        hbox.Add(self.package_input, 1, wx.EXPAND | wx.RIGHT, 5)
        btn_launch_app = wx.Button(self.control_panel, label="启动APP")
        btn_launch_app.Bind(wx.EVT_BUTTON, self.on_launch_app)
        hbox.Add(btn_launch_app, 0)
        vbox.Add(hbox, 0, wx.EXPAND | wx.ALL, 5)
        
        btn_get_current_app = wx.Button(self.control_panel, label="获取前台APP")
        btn_get_current_app.Bind(wx.EVT_BUTTON, self.on_get_current_app)
        vbox.Add(btn_get_current_app, 0, wx.EXPAND | wx.ALL, 5)
        
        self.control_panel.SetSizer(vbox)
        
    def on_get_resolution(self, event):
        """获取分辨率"""
        try:
            width, height = self.device.screen_size
            self.log(f"设备分辨率: {width}x{height}")
        except Exception as e:
            self.log(f"获取分辨率失败: {e}")
            
    def on_get_orientation(self, event):
        """获取设备方向"""
        try:
            orientation = self.device.detect_orientation()
            orientation_text = "横屏" if orientation == "landscape" else "竖屏"
            self.log(f"设备方向: {orientation_text}")
        except Exception as e:
            self.log(f"获取设备方向失败: {e}")
            
    def on_launch_app(self, event):
        """启动APP"""
        package_name = self.package_input.GetValue().strip()
        if not package_name:
            self.log("请输入包名")
            return
        try:
            self.device.launch_app(package_name)
            self.log(f"启动APP: {package_name}")
        except Exception as e:
            self.log(f"启动APP失败: {e}")
            
    def on_get_current_app(self, event):
        """获取前台APP"""
        try:
            package = self.device.current_package()
            if package:
                self.log(f"前台APP: {package}")
            else:
                self.log("获取前台APP失败")
        except Exception as e:
            self.log(f"获取前台APP失败: {e}")
            
    def on_quit(self, event):
        self.device_panel.is_running = False
        self.Close()

def show_device_mirror(device: Device):
    """显示设备镜像窗口"""
    app = wx.App()
    frame = DeviceMirrorFrame(device)
    frame.Show()
    app.MainLoop()

if __name__ == "__main__":
    # 测试代码
    from kotonebot.client.implements.adb import AdbImpl
    from kotonebot.client.implements.uiautomator2 import UiAutomator2Impl
    from adbutils import adb
    
    print("server version:", adb.server_version())
    adb.connect("127.0.0.1:5555")
    print("devices:", adb.device_list())
    d = adb.device_list()[-1]
    
    dd = Device(d)
    adb_imp = AdbImpl(dd)
    dd._command = adb_imp
    dd._touch = adb_imp
    dd._screenshot = UiAutomator2Impl(dd)
    
    show_device_mirror(dd)



from .backend.context import (
    ContextOcr,
    ContextImage,
    ContextDebug,
    ContextColor,
    device,
    ocr,
    image,
    debug,
    color,
    config,
    rect_expand,
    sleep,
    task,
    action,
    use_screenshot,
    wait
)
from .backend.util import (
    Rect,
    grayscaled,
    grayscale_cached,
    cropped,
    AdaptiveWait,
    Countdown,
    Interval,
    until,
    crop_rect,
)
from .backend.color import (
    hsv_cv2web,
    hsv_web2cv,
    rgb_to_hsv,
    hsv_to_rgb
)
from .backend.ocr import (
    fuzz,
    regex,
    contains,
    equals,
)
from .ui import user



import os
import io
import zipfile
import pkgutil
import logging
import importlib
import traceback
import threading
from datetime import datetime
from typing import Callable, Optional, Any, Literal
from dataclasses import dataclass, field

import cv2

from kotonebot.backend.context import init_context, vars
from kotonebot.backend.context import task_registry, action_registry, current_callstack, Task, Action

log_stream = io.StringIO()
stream_handler = logging.StreamHandler(log_stream)
stream_handler.setFormatter(logging.Formatter('[%(asctime)s] [%(levelname)s] [%(name)s] [%(filename)s:%(lineno)d] - %(message)s'))
logging.getLogger('kotonebot').addHandler(stream_handler)
logger = logging.getLogger(__name__)

@dataclass
class TaskStatus:
    task: Task
    status: Literal['pending', 'running', 'finished', 'error', 'cancelled']

@dataclass
class RunStatus:
    running: bool = False
    tasks: list[TaskStatus] = field(default_factory=list)
    current_task: Task | None = None
    callstack: list[Task | Action] = field(default_factory=list)

    def interrupt(self):
        vars.interrupted.set()

def initialize(module: str):
    """
    初始化并载入所有任务和动作。

    :param module: 主模块名。此模块及其所有子模块都会被载入。
    """
    logger.info('Initializing tasks and actions...')
    logger.debug(f'Loading module: {module}')
    # 加载主模块
    importlib.import_module(module)

    # 加载所有子模块
    pkg = importlib.import_module(module)
    for loader, name, is_pkg in pkgutil.walk_packages(pkg.__path__, pkg.__name__ + '.'):
        logger.debug(f'Loading sub-module: {name}')
        try:
            importlib.import_module(name)
        except Exception as e:
            logger.error(f'Failed to load sub-module: {name}')
            logger.exception(f'Error: ')
    
    logger.info('Tasks and actions initialized.')
    logger.info(f'{len(task_registry)} task(s) and {len(action_registry)} action(s) loaded.')

def _save_error_report(
    exception: Exception,
    *,
    path: str | None = None
) -> str:
    """
    保存错误报告

    :param path: 保存的路径。若为 `None`，则保存到 `./reports/{YY-MM-DD HH-MM-SS}.zip`。
    :return: 保存的路径
    """
    from kotonebot import device
    try:
        if path is None:
            path = f'./reports/{datetime.now().strftime("%Y-%m-%d %H-%M-%S")}.zip'
        exception_msg = '\n'.join(traceback.format_exception(exception))
        task_callstack = '\n'.join([f'{i+1}. name={task.name} priority={task.priority}' for i, task in enumerate(current_callstack)])
        screenshot = device.screenshot()
        logs = log_stream.getvalue()
        with open('config.json', 'r', encoding='utf-8') as f:
            config_content = f.read()

        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        with zipfile.ZipFile(path, 'w') as zipf:
            zipf.writestr('exception.txt', exception_msg)
            zipf.writestr('task_callstack.txt', task_callstack)
            zipf.writestr('screenshot.png', cv2.imencode('.png', screenshot)[1].tobytes())
            zipf.writestr('config.json', config_content)
            zipf.writestr('logs.txt', logs)
        return path
    except Exception as e:
        logger.exception(f'Failed to save error report:')
        return ''


def run(
    *,
    debug: bool = False,
    resume_on_error: bool = False,
    config_type: type = dict[str, Any],
    on_finished: Optional[Callable[[], None]] = None,
    on_task_status_changed: Optional[Callable[[Task, Literal['pending', 'running', 'finished', 'error', 'cancelled']], None]] = None,
    on_task_error: Optional[Callable[[Task, Exception], None]] = None,
    auto_save_error_report: bool = True,
):
    """
    按优先级顺序运行所有任务。

    :param debug: 是否为调试模式。调试模式下，不捕获异常，不保存错误报告。默认为 `False`。
    :param resume_on_error: 是否在任务出错时继续运行。默认为 `False`。
    :param auto_save_error_report: 是否自动保存错误报告。默认 `True`。
    """
    # TODO: 允许在 initialize 时指定 config_type。
    # TODO: 允许 init_context 时先不连接设备，而是可以之后第一次截图时连接
    init_context(config_type=config_type)

    tasks = sorted(task_registry.values(), key=lambda x: x.priority, reverse=True)
    for task in tasks:
        if on_task_status_changed:
            on_task_status_changed(task, 'pending')

    for task in tasks:
        logger.info(f'Task started: {task.name}')
        if on_task_status_changed:
            on_task_status_changed(task, 'running')

        if debug:
            task.func()
        else:
            try:
                task.func()
                if on_task_status_changed:
                    on_task_status_changed(task, 'finished')
            # 用户中止
            except KeyboardInterrupt as e:
                logger.exception('Keyboard interrupt detected.')
                for task1 in tasks[tasks.index(task):]:
                    if on_task_status_changed:
                        on_task_status_changed(task1, 'cancelled')
                vars.interrupted.clear()
                break
            # 其他错误
            except Exception as e:
                logger.error(f'Task failed: {task.name}')
                logger.exception(f'Error: ')
                report_path = None
                if auto_save_error_report:
                    report_path = _save_error_report(e)
                if on_task_status_changed:
                    on_task_status_changed(task, 'error')
                if not resume_on_error:
                    for task1 in tasks[tasks.index(task)+1:]:
                        if on_task_status_changed:
                            on_task_status_changed(task1, 'cancelled')
                    break
        logger.info(f'Task finished: {task.name}')
    logger.info('All tasks finished.')
    if on_finished:
        on_finished()

def start(
    *,
    debug: bool = False,
    resume_on_error: bool = False,
    config_type: type = dict[str, Any],
) -> RunStatus:
    run_status = RunStatus(running=True)
    def _on_finished():
        run_status.running = False
        run_status.current_task = None
        run_status.callstack = []
    def _on_task_status_changed(task: Task, status: Literal['pending', 'running', 'finished', 'error']):
        def _find(task: Task) -> TaskStatus:
            for task_status in run_status.tasks:
                if task_status.task == task:
                    return task_status
            raise ValueError(f'Task {task.name} not found in run_status.tasks')
        if status == 'pending':
            run_status.tasks.append(TaskStatus(task=task, status='pending'))
        else:
            _find(task).status = status
    thread = threading.Thread(target=run, kwargs={
        'config_type': config_type,
        'debug': debug,
        'resume_on_error': resume_on_error,
        'on_finished': _on_finished,
        'on_task_status_changed': _on_task_status_changed,
    })
    thread.start()
    return run_status

def execute(task: Task, config_type: type = dict[str, Any]):
    """
    执行某个任务。

    :param task: 任务。
    :param config_type: 配置类型。
    """
    init_context(config_type=config_type)
    initialize('kotonebot.tasks')
    task.func()

if __name__ == '__main__':
    from kotonebot.tasks.common import BaseConfig
    from kotonebot.backend.util import Profiler
    logging.basicConfig(level=logging.INFO, format='[%(asctime)s] [%(levelname)s] [%(name)s] [%(filename)s:%(lineno)d] - %(message)s')
    logger.setLevel(logging.DEBUG)
    logging.getLogger('kotonebot').setLevel(logging.DEBUG)
    init_context(config_type=BaseConfig)
    initialize('kotonebot.tasks')
    pf = Profiler('profiler')
    pf.begin()
    run()
    pf.end()
    pf.snakeviz()





"""从商店购买物品"""
import logging
from typing import Optional
from typing_extensions import deprecated

from . import R
from .common import conf, DailyMoneyShopItems
from kotonebot.backend.util import cropped
from kotonebot import task, device, image, ocr, action, sleep
from kotonebot.backend.dispatch import DispatcherContext, SimpleDispatcher, dispatcher
from .actions.scenes import goto_home, goto_shop, at_daily_shop

logger = logging.getLogger(__name__)

@action('购买 Money 物品', screenshot_mode='manual-inherit')
def money_items2(items: Optional[list[DailyMoneyShopItems]] = None):
    """
    购买 Money 物品

    前置条件：商店页面的 マニー Tab\n
    结束状态：-

    :param items: 要购买的物品列表，默认为 None。为 None 时使用配置文件里的设置。
    """
    # 前置条件：[screenshots\shop\money1.png]
    logger.info(f'Purchasing マニー items.')

    if items is None:
        items = conf().purchase.money_items
    
    device.screenshot()
    if DailyMoneyShopItems.Recommendations in items:
        dispatch_recommended_items()
        items.remove(DailyMoneyShopItems.Recommendations)

    finished = []
    max_scroll = 3
    scroll = 0
    while items:
        for item in items:
            if image.find(item.to_resource(), colored=True):
                logger.info(f'Purchasing {item.to_ui_text(item)}...')
                device.click()
                handle_purchase_dialog()
                finished.append(item)
        items = [item for item in items if item not in finished]
        # 全都买完了
        if not items:
            break
        # 还有，翻页后继续
        else:
            device.swipe_scaled(x1=0.5, x2=0.5, y1=0.8, y2=0.5)
            sleep(0.5)
            device.screenshot()
            scroll += 1
            if scroll >= max_scroll:
                break
    logger.info(f'Purchasing money items completed. {len(finished)} item(s) purchased.')
    if items:
        logger.info(f'{len(items)} item(s) not purchased/already purchased: {", ".join([item.to_ui_text(item) for item in items])}')

@action('购买推荐商品', screenshot_mode='manual-inherit')
def dispatch_recommended_items():
    """
    购买推荐商品

    前置条件：商店页面的 マニー Tab\n
    结束状态：-
    """
    # 前置条件：[screenshots\shop\money1.png]
    logger.info(f'Start purchasing recommended items.')

    while True:
        device.screenshot()
        if image.find(R.Daily.TextShopRecommended):
            logger.info(f'Clicking on recommended item.') # TODO: 计数
            device.click()
            handle_purchase_dialog()
        elif image.find(R.Daily.IconTitleDailyShop) and not image.find(R.Daily.TextShopRecommended):
            logger.info(f'No recommended item found. Finished.')
            break

@action('确认购买', screenshot_mode='manual-inherit')
def handle_purchase_dialog():
    """
    确认购买

    前置条件：点击某个商品后的瞬间\n
    结束状态：对话框关闭后原来的界面
    """
    # 前置条件：[screenshots\shop\dialog.png]
    # TODO: 需要有个更好的方式检测是否已购买
    purchased = (SimpleDispatcher('dispatch_purchase_dialog')
        .until(R.Common.ButtonConfirm, result=False)
        .until(R.Daily.TextShopPurchased, result=True)
        .timeout(timeout=3, result=True)
    ).run()
    
    if purchased:
        logger.info('Item sold out.')
        sleep(1) # 等待售罄提示消失
        return
    else:
        device.screenshot()
        while image.find(R.Daily.ButtonShopCountAdd, colored=True):
            logger.debug('Adjusting quantity(+1)...')
            device.click()
            sleep(0.2)
            device.screenshot()
        logger.debug('Confirming purchase...')
        device.click(image.expect_wait(R.Common.ButtonConfirm))
    # 等待对话框动画结束
    image.expect_wait(R.Daily.IconTitleDailyShop)

@action('购买 AP 物品')
def ap_items():
    """
    购买 AP 物品

    前置条件：位于商店页面的 AP Tab
    """
    # [screenshots\shop\ap1.png]
    logger.info(f'Purchasing AP items.')
    results = image.find_all(R.Daily.IconShopAp, threshold=0.7)
    sleep(1)
    # 按 X, Y 坐标排序从小到大
    results = sorted(results, key=lambda x: (x.position[0], x.position[1]))
    # 按照配置文件里的设置过滤
    item_indices = conf().purchase.ap_items
    logger.info(f'Purchasing AP items: {item_indices}')
    for index in item_indices:
        if index <= len(results):
            logger.info(f'Purchasing #{index} AP item.')
            device.click(results[index])
            sleep(0.5)
            with cropped(device, y1=0.3):
                purchased = image.wait_for(R.Daily.TextShopPurchased, timeout=1)
                if purchased is not None:
                    logger.info(f'AP item #{index} already purchased.')
                    continue
                comfirm = image.expect_wait(R.Common.ButtonConfirm, timeout=2)
                # 如果数量不是最大,调到最大
                while image.find(R.Daily.ButtonShopCountAdd, colored=True):
                    logger.debug('Adjusting quantity(+1)...')
                    device.click()
                    sleep(0.3)
                logger.debug(f'Confirming purchase...')
                device.click(comfirm)
                sleep(1.5)
        else:
            logger.warning(f'AP item #{index} not found')
    logger.info(f'Purchasing AP items completed. {len(item_indices)} items purchased.')

@task('商店购买')
def purchase():
    """
    从商店购买物品
    """
    if not conf().purchase.enabled:
        logger.info('Purchase is disabled.')
        return
    if not at_daily_shop():
        goto_shop()
    # 进入每日商店 [screenshots\shop\shop.png]
    device.click(image.expect(R.Daily.ButtonDailyShop)) # TODO: memoable
    sleep(1)

    # 购买マニー物品
    if conf().purchase.money_enabled:
        image.expect_wait(R.Daily.IconShopMoney)
        money_items2()
        sleep(0.5)
    else:
        logger.info('Money purchase is disabled.')
    
    # 购买 AP 物品
    if conf().purchase.ap_enabled:
        # 点击 AP 选项卡
        device.click(image.expect_wait(R.Daily.TextTabShopAp, timeout=2)) # TODO: memoable
        # 等待 AP 选项卡加载完成
        image.expect_wait(R.Daily.IconShopAp)
        ap_items()
        sleep(0.5)
    else:
        logger.info('AP purchase is disabled.')
    
    goto_home()

if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.INFO, format='[%(asctime)s] [%(levelname)s] [%(name)s] [%(funcName)s] [%(lineno)d] %(message)s')
    logger.setLevel(logging.DEBUG)
    purchase()



class UnrecoverableError(Exception):
    pass

class ResourceFileMissingError(Exception):
    def __init__(self, file_path: str, description: str):
        self.file_path = file_path
        self.description = description
        super().__init__(f'Resource file ({description}) "{file_path}" is missing.')

class KotonebotWarning(Warning):
    pass